@@ -40,7 +40,6 @@
 #include "logger.h"
 #include "conffile.h"
 #include "cache.h"
-#include "player.h"
 #include "http.h"
 
 #include "avio_evbuffer.h"
@@ -145,6 +144,7 @@ static int source_item_embedded_get(struct artwork_ctx *ctx);
 static int source_item_own_get(struct artwork_ctx *ctx);
 static int source_item_stream_get(struct artwork_ctx *ctx);
 static int source_item_spotify_get(struct artwork_ctx *ctx);
+static int source_item_ownpl_get(struct artwork_ctx *ctx);
 
 /* List of sources that can provide artwork for a group (i.e. usually an album
  * identified by a persistentid). The source handlers will be called in the
@@ -206,6 +206,12 @@ static struct artwork_source artwork_item_source[] =
       .data_kinds = (1 << DATA_KIND_SPOTIFY),
       .cache = ON_SUCCESS,
     },
+    {
+      .name = "playlist own",
+      .handler = source_item_ownpl_get,
+      .data_kinds = (1 << DATA_KIND_HTTP),
+      .cache = ON_SUCCESS | ON_FAILURE,
+    },
     {
       .name = NULL,
       .handler = NULL,
@@ -450,7 +456,7 @@ artwork_rescale(struct evbuffer *evbuf, AVFormatContext *src_ctx, int s, int out
   dst->codec_id = dst_fmt->video_codec;
   dst->codec_type = AVMEDIA_TYPE_VIDEO;
 
-  dst->pix_fmt = avcodec_find_best_pix_fmt_of_list((enum AVPixelFormat *)img_encoder->pix_fmts, src->pix_fmt, 1, NULL);
+  dst->pix_fmt = avcodec_default_get_format(dst, img_encoder->pix_fmts);
   if (dst->pix_fmt < 0)
     {
       DPRINTF(E_LOG, L_ART, "Could not determine best pixel format\n");
@@ -498,11 +504,14 @@ artwork_rescale(struct evbuffer *evbuf, AVFormatContext *src_ctx, int s, int out
       goto out_free_frames;
     }
 
-#ifdef HAVE_LIBAV_IMAGE_FILL_ARRAYS
+#if HAVE_DECL_AV_IMAGE_FILL_ARRAYS
   av_image_fill_arrays(o_frame->data, o_frame->linesize, buf, dst->pix_fmt, src->width, src->height, 1);
 #else
   avpicture_fill((AVPicture *)o_frame, buf, dst->pix_fmt, src->width, src->height);
 #endif
+  o_frame->height = dst->height;
+  o_frame->width = dst->width;
+  o_frame->format = dst->pix_fmt;
 
   swsctx = sws_getContext(src->width, src->height, src->pix_fmt,
 			  dst->width, dst->height, dst->pix_fmt,
@@ -1094,6 +1103,7 @@ static int
 source_item_stream_get(struct artwork_ctx *ctx)
 {
   struct http_client_ctx client;
+  struct db_queue_item *queue_item;
   struct keyval *kv;
   const char *content_type;
   char *url;
@@ -1105,9 +1115,15 @@ source_item_stream_get(struct artwork_ctx *ctx)
 
   ret = ART_E_NONE;
 
-  url = player_get_icy_artwork_url(ctx->id);
-  if (!url)
-    return ART_E_NONE;
+  queue_item = db_queue_fetch_byfileid(ctx->id);
+  if (!queue_item || !queue_item->artwork_url)
+    {
+      free_queue_item(queue_item, 0);
+      return ART_E_NONE;
+    }
+
+  url = strdup(queue_item->artwork_url);
+  free_queue_item(queue_item, 0);
 
   len = strlen(url);
   if ((len < 14) || (len > PATH_MAX)) // Can't be shorter than http://a/1.jpg
@@ -1129,8 +1145,8 @@ source_item_stream_get(struct artwork_ctx *ctx)
 
   memset(&client, 0, sizeof(struct http_client_ctx));
   client.url = url;
-  client.headers = kv;
-  client.body = ctx->evbuf;
+  client.input_headers = kv;
+  client.input_body = ctx->evbuf;
 
   if (http_client_request(&client) < 0)
     goto out_kv;
@@ -1285,6 +1301,60 @@ source_item_spotify_get(struct artwork_ctx *ctx)
 }
 #endif
 
+/* First looks of the mfi->path is in any playlist, and if so looks in the dir
+ * of the playlist file (m3u et al) to see if there is any artwork. So if the
+ * playlist is /foo/bar.m3u it will look for /foo/bar.png and /foo/bar.jpg.
+ */
+static int
+source_item_ownpl_get(struct artwork_ctx *ctx)
+{
+  struct query_params qp;
+  struct db_playlist_info dbpli;
+  char filter[PATH_MAX + 64];
+  char *mfi_path;
+  int format;
+  int ret;
+
+  ret = snprintf(filter, sizeof(filter), "(filepath = '%s')", ctx->dbmfi->path);
+  if ((ret < 0) || (ret >= sizeof(filter)))
+    {
+      DPRINTF(E_LOG, L_ART, "Artwork path exceeds PATH_MAX (%s)\n", ctx->dbmfi->path);
+      return ART_E_ERROR;
+    }
+
+  memset(&qp, 0, sizeof(struct query_params));
+  qp.type = Q_FIND_PL;
+  qp.filter = filter;
+
+  ret = db_query_start(&qp);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_ART, "Could not start ownpl query\n");
+      return ART_E_ERROR;
+    }
+
+  mfi_path = ctx->dbmfi->path;
+
+  format = ART_E_NONE;
+  while (((ret = db_query_fetch_pl(&qp, &dbpli, 0)) == 0) && (dbpli.id) && (format == ART_E_NONE))
+    {
+      if (!dbpli.path)
+	continue;
+
+      ctx->dbmfi->path = dbpli.path;
+      format = source_item_own_get(ctx);
+    }
+
+  ctx->dbmfi->path = mfi_path;
+
+  if ((ret < 0) || (format < 0))
+    format = ART_E_ERROR;
+
+  db_query_end(&qp);
+
+  return format;
+}
+
 
 /* --------------------------- SOURCE PROCESSING --------------------------- */
 
