@@ -56,6 +56,11 @@ static cfg_opt_t sec_general[] =
     CFG_STR("cache_path", STATEDIR "/cache/" PACKAGE "/cache.db", CFGF_NONE),
     CFG_INT("cache_daap_threshold", 1000, CFGF_NONE),
     CFG_BOOL("speaker_autoselect", cfg_true, CFGF_NONE),
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+    CFG_BOOL("high_resolution_clock", cfg_false, CFGF_NONE),
+#else
+    CFG_BOOL("high_resolution_clock", cfg_true, CFGF_NONE),
+#endif
     CFG_STR("allow_origin", "*", CFGF_NONE),
     CFG_END()
   };
@@ -82,13 +87,14 @@ static cfg_opt_t sec_library[] =
     CFG_STR("name_radio", "Radio", CFGF_NONE),
     CFG_STR_LIST("artwork_basenames", "{artwork,cover,Folder}", CFGF_NONE),
     CFG_BOOL("artwork_individual", cfg_false, CFGF_NONE),
-    CFG_STR_LIST("filetypes_ignore", "{.db,.ini,.db-journal,.pdf}", CFGF_NONE),
+    CFG_STR_LIST("filetypes_ignore", "{.db,.ini,.db-journal,.pdf,.metadata}", CFGF_NONE),
     CFG_STR_LIST("filepath_ignore", NULL, CFGF_NONE),
     CFG_BOOL("filescan_disable", cfg_false, CFGF_NONE),
     CFG_BOOL("itunes_overrides", cfg_false, CFGF_NONE),
     CFG_BOOL("itunes_smartpl", cfg_false, CFGF_NONE),
     CFG_STR_LIST("no_decode", NULL, CFGF_NONE),
     CFG_STR_LIST("force_decode", NULL, CFGF_NONE),
+    CFG_BOOL("pipe_autostart", cfg_true, CFGF_NONE),
     CFG_END()
   };
 
@@ -108,6 +114,7 @@ static cfg_opt_t sec_audio[] =
 static cfg_opt_t sec_airplay[] =
   {
     CFG_INT("max_volume", 11, CFGF_NONE),
+    CFG_BOOL("exclude", cfg_false, CFGF_NONE),
     CFG_STR("password", NULL, CFGF_NONE),
     CFG_END()
   };
