@@ -8,6 +8,7 @@
 
 #include <sqlite3.h>
 
+#include "outputs.h"
 
 enum index_type {
   I_NONE,
@@ -28,6 +29,8 @@ enum sort_type {
   S_DISC,
   S_TRACK,
   S_VPATH,
+  S_POS,
+  S_SHUFFLE_POS,
 };
 
 #define Q_F_BROWSE (1 << 15)
@@ -35,21 +38,22 @@ enum sort_type {
 enum query_type {
   Q_ITEMS            = 1,
   Q_PL               = 2,
-  Q_PLITEMS          = 3,
-  Q_BROWSE_ARTISTS   = Q_F_BROWSE | 4,
-  Q_BROWSE_ALBUMS    = Q_F_BROWSE | 5,
-  Q_BROWSE_GENRES    = Q_F_BROWSE | 6,
-  Q_BROWSE_COMPOSERS = Q_F_BROWSE | 7,
-  Q_GROUP_ALBUMS     = 8,
-  Q_GROUP_ARTISTS    = 9,
-  Q_GROUP_ITEMS      = 10,
-  Q_GROUP_DIRS       = Q_F_BROWSE | 11,
-  Q_BROWSE_YEARS     = Q_F_BROWSE | 12,
-  Q_COUNT_ITEMS      = 13,
-  Q_BROWSE_DISCS     = Q_F_BROWSE | 14,
-  Q_BROWSE_TRACKS    = Q_F_BROWSE | 15,
-  Q_BROWSE_VPATH     = Q_F_BROWSE | 16,
-  Q_BROWSE_PATH      = Q_F_BROWSE | 17,
+  Q_FIND_PL          = 3,
+  Q_PLITEMS          = 4,
+  Q_GROUP_ALBUMS     = 5,
+  Q_GROUP_ARTISTS    = 6,
+  Q_GROUP_ITEMS      = 7,
+  Q_COUNT_ITEMS      = 8,
+  Q_BROWSE_ARTISTS   = Q_F_BROWSE | 1,
+  Q_BROWSE_ALBUMS    = Q_F_BROWSE | 2,
+  Q_BROWSE_GENRES    = Q_F_BROWSE | 3,
+  Q_BROWSE_COMPOSERS = Q_F_BROWSE | 4,
+  Q_GROUP_DIRS       = Q_F_BROWSE | 5,
+  Q_BROWSE_YEARS     = Q_F_BROWSE | 6,
+  Q_BROWSE_DISCS     = Q_F_BROWSE | 7,
+  Q_BROWSE_TRACKS    = Q_F_BROWSE | 8,
+  Q_BROWSE_VPATH     = Q_F_BROWSE | 9,
+  Q_BROWSE_PATH      = Q_F_BROWSE | 10,
 };
 
 #define ARTWORK_UNKNOWN   0
@@ -189,7 +193,7 @@ struct media_file_info {
 
 #define mfi_offsetof(field) offsetof(struct media_file_info, field)
 
-/* PL_SPECIAL value must be in sync with type value in Q_PL* in db.c */
+/* PL_SPECIAL value must be in sync with type value in Q_PL* in db_init.c */
 enum pl_type {
   PL_SPECIAL = 0,
   PL_FOLDER = 1,
@@ -347,15 +351,16 @@ struct watch_enum {
 
 struct filecount_info {
   uint32_t count;
-  uint32_t length;
+  uint64_t length;
 };
 
-/* Directory ids must be in sync with the ids in Q_DIR* in db.c */
+/* Directory ids must be in sync with the ids in Q_DIR* in db_init.c */
 enum directory_ids {
   DIR_ROOT = 1,
   DIR_FILE = 2,
   DIR_HTTP = 3,
   DIR_SPOTIFY = 4,
+  DIR_MAX
 };
 
 struct directory_info {
@@ -373,6 +378,51 @@ struct directory_enum {
   sqlite3_stmt *stmt;
 };
 
+struct db_queue_item
+{
+  /* A unique id for this queue item. If the same item appears multiple
+     times in the queue each corresponding queue item has its own id. */
+  uint32_t id;
+
+  /* Id of the file/item in the files database */
+  uint32_t file_id;
+
+  /* Length of the item in ms */
+  uint32_t song_length;
+
+  /* Data type of the item */
+  enum data_kind data_kind;
+  /* Media type of the item */
+  enum media_kind media_kind;
+
+  uint32_t seek;
+
+  uint32_t pos;
+  uint32_t shuffle_pos;
+
+  char *path;
+  char *virtual_path;
+
+  char *title;
+  char *artist;
+  char *album_artist;
+  char *album;
+  char *genre;
+
+  int64_t songalbumid;
+  uint32_t time_modified;
+
+  char *artist_sort;
+  char *album_sort;
+  char *album_artist_sort;
+
+  uint32_t year;
+  uint32_t track;
+  uint32_t disc;
+
+  char *artwork_url;
+};
+
 char *
 db_escape_string(const char *str);
 
@@ -391,6 +441,9 @@ free_pli(struct playlist_info *pli, int content_only);
 void
 free_di(struct directory_info *di, int content_only);
 
+void
+free_queue_item(struct db_queue_item *queue_item, int content_only);
+
 /* Maintenance and DB hygiene */
 void
 db_hook_post_scan(void);
@@ -408,6 +461,9 @@ db_transaction_begin(void);
 void
 db_transaction_end(void);
 
+void
+db_transaction_rollback(void);
+
 /* Queries */
 int
 db_query_start(struct query_params *qp);
@@ -446,12 +502,6 @@ db_files_get_album_count(void);
 int
 db_files_get_count_bymatch(char *path);
 
-void
-db_files_update_songartistid(void);
-
-void
-db_files_update_songalbumid(void);
-
 void
 db_file_inc_playcount(int id);
 
@@ -470,9 +520,6 @@ db_file_id_bypath(char *path);
 int
 db_file_id_bymatch(char *path);
 
-int
-db_file_id_byfilebase(char *filename, char *base);
-
 int
 db_file_id_byfile(char *filename);
 
@@ -483,7 +530,7 @@ int
 db_file_id_by_virtualpath_match(char *path);
 
 void
-db_file_stamp_bypath(char *path, time_t *stamp, int *id);
+db_file_stamp_bypath(const char *path, time_t *stamp, int *id);
 
 struct media_file_info *
 db_file_fetch_byid(int id);
@@ -498,10 +545,7 @@ int
 db_file_update(struct media_file_info *mfi);
 
 void
-db_file_update_icy(int id, char *artist, char *album);
-
-void
-db_file_save_seek(int id, uint32_t seek);
+db_file_seek_update(int id, uint32_t seek);
 
 void
 db_file_delete_bypath(char *path);
@@ -529,7 +573,7 @@ void
 db_pl_ping_bymatch(char *path, int isdir);
 
 struct playlist_info *
-db_pl_fetch_bypath(char *path);
+db_pl_fetch_bypath(const char *path);
 
 struct playlist_info *
 db_pl_fetch_byvirtualpath(char *virtual_path);
@@ -541,7 +585,7 @@ int
 db_pl_add(struct playlist_info *pli, int *id);
 
 int
-db_pl_add_item_bypath(int plid, char *path);
+db_pl_add_item_bypath(int plid, const char *path);
 
 int
 db_pl_add_item_byid(int plid, int fileid);
@@ -569,7 +613,7 @@ db_pl_enable_bycookie(uint32_t cookie, char *path);
 
 /* Groups */
 int
-db_groups_clear(void);
+db_groups_cleanup();
 
 int
 db_group_persistentid_byid(int id, int64_t *persistentid);
@@ -592,7 +636,7 @@ int
 db_directory_addorupdate(char *virtual_path, int disabled, int parent_id);
 
 void
-db_directory_ping_bymatch(char *path);
+db_directory_ping_bymatch(char *virtual_path);
 
 void
 db_directory_disable_bymatch(char *path, char *strip, uint32_t cookie);
@@ -619,32 +663,114 @@ void
 db_spotify_pl_delete(int id);
 
 void
-db_spotify_files_delete();
+db_spotify_files_delete(void);
 #endif
 
 /* Admin */
 int
-db_admin_add(const char *key, const char *value);
+db_admin_set(const char *key, const char *value);
 
 char *
 db_admin_get(const char *key);
 
-int
-db_admin_update(const char *key, const char *value);
-
 int
 db_admin_delete(const char *key);
 
-/* Speakers */
+/* Speakers/outputs */
 int
-db_speaker_save(uint64_t id, int selected, int volume, const char *name);
+db_speaker_save(struct output_device *device);
 
 int
-db_speaker_get(uint64_t id, int *selected, int *volume);
+db_speaker_get(struct output_device *device, uint64_t id);
 
 void
 db_speaker_clear_all(void);
 
+/* Queue */
+int
+db_queue_get_version();
+
+int
+db_queue_update_item(struct db_queue_item *queue_item);
+
+int
+db_queue_add_by_queryafteritemid(struct query_params *qp, uint32_t item_id);
+
+int
+db_queue_add_by_query(struct query_params *qp, char reshuffle, uint32_t item_id);
+
+int
+db_queue_add_by_playlistid(int plid, char reshuffle, uint32_t item_id);
+
+int
+db_queue_add_by_fileid(int id, char reshuffle, uint32_t item_id);
+
+int
+db_queue_add_item(struct db_queue_item *queue_item, char reshuffle, uint32_t item_id);
+
+int
+db_queue_enum_start(struct query_params *qp);
+
+void
+db_queue_enum_end(struct query_params *qp);
+
+int
+db_queue_enum_fetch(struct query_params *qp, struct db_queue_item *queue_item);
+
+struct db_queue_item *
+db_queue_fetch_byitemid(uint32_t item_id);
+
+struct db_queue_item *
+db_queue_fetch_byfileid(uint32_t file_id);
+
+struct db_queue_item *
+db_queue_fetch_bypos(uint32_t pos, char shuffle);
+
+struct db_queue_item *
+db_queue_fetch_byposrelativetoitem(int pos, uint32_t item_id, char shuffle);
+
+struct db_queue_item *
+db_queue_fetch_next(uint32_t item_id, char shuffle);
+
+struct db_queue_item *
+db_queue_fetch_prev(uint32_t item_id, char shuffle);
+
+int
+db_queue_cleanup();
+
+int
+db_queue_clear(uint32_t keep_item_id);
+
+int
+db_queue_delete_byitemid(uint32_t item_id);
+
+int
+db_queue_delete_bypos(uint32_t pos, int count);
+
+int
+db_queue_delete_byposrelativetoitem(uint32_t pos, uint32_t item_id, char shuffle);
+
+int
+db_queue_move_byitemid(uint32_t item_id, int pos_to, char shuffle);
+
+int
+db_queue_move_bypos(int pos_from, int pos_to);
+
+int
+db_queue_move_byposrelativetoitem(uint32_t from_pos, uint32_t to_offset, uint32_t item_id, char shuffle);
+
+int
+db_queue_reshuffle(uint32_t item_id);
+
+int
+db_queue_get_count();
+
+int
+db_queue_get_pos(uint32_t item_id, char shuffle);
+
+int
+db_queue_get_pos_byfileid(uint32_t file_id, char shuffle);
+
 /* Inotify */
 int
 db_watch_clear(void);
