@@ -1,3 +1,4 @@
+
 /*
  * Copyright (C) 2009-2011 Julien BLACHE <jb@jblache.org>
  * Copyright (C) 2010 Kai Elwert <elwertk@googlemail.com>
@@ -25,10 +26,10 @@
 #include "logger.h"
 
 
-#define T_ADMIN					\
-  "CREATE TABLE IF NOT EXISTS admin("		\
-  "   key   VARCHAR(32) NOT NULL,"		\
-  "   value VARCHAR(32) NOT NULL"		\
+#define T_ADMIN						\
+  "CREATE TABLE IF NOT EXISTS admin("			\
+  "   key   VARCHAR(32) PRIMARY KEY NOT NULL,"		\
+  "   value VARCHAR(255) NOT NULL"			\
   ");"
 
 #define T_FILES						\
@@ -139,7 +140,8 @@
   "   id             INTEGER PRIMARY KEY NOT NULL,"	\
   "   selected       INTEGER NOT NULL,"			\
   "   volume         INTEGER NOT NULL,"			\
-  "   name           VARCHAR(255) DEFAULT NULL"         \
+  "   name           VARCHAR(255) DEFAULT NULL,"        \
+  "   auth_key       VARCHAR(2048) DEFAULT NULL"        \
   ");"
 
 #define T_INOTIFY					\
@@ -158,6 +160,33 @@
   "   parent_id           INTEGER DEFAULT 0"			\
   ");"
 
+#define T_QUEUE								\
+  "CREATE TABLE IF NOT EXISTS queue ("					\
+  "   id                  INTEGER PRIMARY KEY NOT NULL,"		\
+  "   file_id             INTEGER NOT NULL,"				\
+  "   pos                 INTEGER NOT NULL,"				\
+  "   shuffle_pos         INTEGER NOT NULL,"				\
+  "   data_kind           INTEGER NOT NULL,"				\
+  "   media_kind          INTEGER NOT NULL,"				\
+  "   song_length         INTEGER NOT NULL,"				\
+  "   path                VARCHAR(4096) NOT NULL,"			\
+  "   virtual_path        VARCHAR(4096) NOT NULL,"			\
+  "   title               VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   artist              VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   album_artist        VARCHAR(1024) NOT NULL COLLATE DAAP,"		\
+  "   album               VARCHAR(1024) NOT NULL COLLATE DAAP,"		\
+  "   genre               VARCHAR(255) DEFAULT NULL COLLATE DAAP,"	\
+  "   songalbumid         INTEGER NOT NULL,"				\
+  "   time_modified       INTEGER DEFAULT 0,"				\
+  "   artist_sort         VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   album_sort          VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   album_artist_sort   VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   year                INTEGER DEFAULT 0,"				\
+  "   track               INTEGER DEFAULT 0,"				\
+  "   disc                INTEGER DEFAULT 0,"				\
+  "   artwork_url         VARCHAR(4096) DEFAULT NULL"			\
+  ");"
+
 #define TRG_GROUPS_INSERT_FILES						\
   "CREATE TRIGGER update_groups_new_file AFTER INSERT ON files FOR EACH ROW" \
   " BEGIN"								\
@@ -216,6 +245,9 @@
   "INSERT INTO directories (id, virtual_path, db_timestamp, disabled, parent_id)" \
   " VALUES (4, '/spotify:', 0, 4294967296, 1);"
 
+#define Q_QUEUE_VERSION			\
+  "INSERT INTO admin (key, value) VALUES ('queue_version', '0');"
+
 #define Q_SCVER_MAJOR					\
   "INSERT INTO admin (key, value) VALUES ('schema_version_major', '%d');"
 #define Q_SCVER_MINOR					\
@@ -237,6 +269,7 @@ static const struct db_init_query db_init_table_queries[] =
     { T_SPEAKERS,  "create table speakers" },
     { T_INOTIFY,   "create table inotify" },
     { T_DIRECTORIES, "create table directories" },
+    { T_QUEUE,     "create table queue" },
 
     { TRG_GROUPS_INSERT_FILES,    "create trigger update_groups_new_file" },
     { TRG_GROUPS_UPDATE_FILES,    "create trigger update_groups_update_file" },
@@ -251,6 +284,7 @@ static const struct db_init_query db_init_table_queries[] =
     { Q_DIR2,      "create default base directory '/file:'" },
     { Q_DIR3,      "create default base directory '/http:'" },
     { Q_DIR4,      "create default base directory '/spotify:'" },
+    { Q_QUEUE_VERSION, "initialize queue version" },
   };
 
 
@@ -327,6 +361,12 @@ static const struct db_init_query db_init_table_queries[] =
 #define I_DIR_PARENT				\
   "CREATE INDEX IF NOT EXISTS idx_dir_parentid ON directories(parent_id);"
 
+#define I_QUEUE_POS				\
+  "CREATE INDEX IF NOT EXISTS idx_queue_pos ON queue(pos);"
+
+#define I_QUEUE_SHUFFLEPOS				\
+  "CREATE INDEX IF NOT EXISTS idx_queue_shufflepos ON queue(shuffle_pos);"
+
 static const struct db_init_query db_init_index_queries[] =
   {
     { I_RESCAN,    "create rescan index" },
@@ -357,6 +397,9 @@ static const struct db_init_query db_init_index_queries[] =
 
     { I_DIR_VPATH,   "create directories disabled_virtualpath index" },
     { I_DIR_PARENT,  "create directories parentid index" },
+
+    { I_QUEUE_POS,  "create queue pos index" },
+    { I_QUEUE_SHUFFLEPOS,  "create queue shuffle pos index" },
   };
 
 int
