@@ -377,7 +377,7 @@ db_upgrade_v11(sqlite3 *hdl)
   else if (count < 0)
     return -1;
 
-  spkids = (uint64_t *)malloc(count * sizeof(uint64_t));
+  spkids = calloc(count, sizeof(uint64_t));
   if (!spkids)
     {
       DPRINTF(E_LOG, L_DB, "Out of memory for speaker IDs\n");
@@ -1073,7 +1073,6 @@ db_upgrade_v16(sqlite3 *hdl)
 	}
     }
 
-  sqlite3_free(errmsg);
   sqlite3_finalize(stmt);
 
   return 0;
@@ -1442,6 +1441,129 @@ db_upgrade_v19(sqlite3 *hdl)
   return 0;
 }
 
+/* Upgrade from schema v19.00 to v19.01 */
+/* Create new table queue for persistent playqueue
+ */
+
+#define U_V1901_CREATE_TABLE_QUEUE					\
+  "CREATE TABLE IF NOT EXISTS queue ("					\
+  "   id                  INTEGER PRIMARY KEY NOT NULL,"		\
+  "   file_id             INTEGER NOT NULL,"				\
+  "   pos                 INTEGER NOT NULL,"				\
+  "   shuffle_pos         INTEGER NOT NULL,"				\
+  "   data_kind           INTEGER NOT NULL,"				\
+  "   media_kind          INTEGER NOT NULL,"				\
+  "   song_length         INTEGER NOT NULL,"				\
+  "   path                VARCHAR(4096) NOT NULL,"			\
+  "   virtual_path        VARCHAR(4096) NOT NULL,"			\
+  "   title               VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   artist              VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   album_artist        VARCHAR(1024) NOT NULL COLLATE DAAP,"		\
+  "   album               VARCHAR(1024) NOT NULL COLLATE DAAP,"		\
+  "   genre               VARCHAR(255) DEFAULT NULL COLLATE DAAP,"	\
+  "   songalbumid         INTEGER NOT NULL,"				\
+  "   time_modified       INTEGER DEFAULT 0,"				\
+  "   artist_sort         VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   album_sort          VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   album_artist_sort   VARCHAR(1024) DEFAULT NULL COLLATE DAAP,"	\
+  "   year                INTEGER DEFAULT 0,"				\
+  "   track               INTEGER DEFAULT 0,"				\
+  "   disc                INTEGER DEFAULT 0"				\
+  ");"
+
+#define U_V1901_QUEUE_VERSION			\
+  "INSERT INTO admin (key, value) VALUES ('queue_version', '0');"
+
+#define U_V1901_SCVER_MAJOR			\
+  "UPDATE admin SET value = '19' WHERE key = 'schema_version_major';"
+#define U_V1901_SCVER_MINOR			\
+  "UPDATE admin SET value = '01' WHERE key = 'schema_version_minor';"
+
+static const struct db_upgrade_query db_upgrade_v1901_queries[] =
+  {
+    { U_V1901_CREATE_TABLE_QUEUE,    "create table directories" },
+    { U_V1901_QUEUE_VERSION,         "insert queue version" },
+
+    { U_V1901_SCVER_MAJOR,    "set schema_version_major to 19" },
+    { U_V1901_SCVER_MINOR,    "set schema_version_minor to 01" },
+  };
+
+/* Upgrade from schema v19.01 to v19.02 */
+/* Set key column as primary key in the admin table
+ */
+
+#define U_V1902_CREATE_TABLE_ADMINTMP 			\
+  "CREATE TEMPORARY TABLE IF NOT EXISTS admin_tmp("	\
+  "   key   VARCHAR(32) NOT NULL,"		\
+  "   value VARCHAR(32) NOT NULL"			\
+  ");"
+#define U_V1902_INSERT_ADMINTMP \
+  "INSERT INTO admin_tmp SELECT * FROM admin;"
+#define U_V1902_DROP_TABLE_ADMIN \
+  "DROP TABLE admin;"
+#define U_V1902_CREATE_TABLE_ADMIN 			\
+  "CREATE TABLE IF NOT EXISTS admin("			\
+  "   key   VARCHAR(32) PRIMARY KEY NOT NULL,"		\
+  "   value VARCHAR(32) NOT NULL"			\
+  ");"
+#define U_V1902_INSERT_ADMIN \
+  "INSERT OR IGNORE INTO admin SELECT * FROM admin_tmp;"
+#define U_V1902_DROP_TABLE_ADMINTMP \
+  "DROP TABLE admin_tmp;"
+
+#define U_V1902_SCVER_MAJOR			\
+  "UPDATE admin SET value = '19' WHERE key = 'schema_version_major';"
+#define U_V1902_SCVER_MINOR			\
+  "UPDATE admin SET value = '02' WHERE key = 'schema_version_minor';"
+
+static const struct db_upgrade_query db_upgrade_v1902_queries[] =
+  {
+    { U_V1902_CREATE_TABLE_ADMINTMP,    "create temporary table admin_tmp" },
+    { U_V1902_INSERT_ADMINTMP,          "insert admin_tmp" },
+    { U_V1902_DROP_TABLE_ADMIN,         "drop table admin" },
+    { U_V1902_CREATE_TABLE_ADMIN,       "create table admin" },
+    { U_V1902_INSERT_ADMIN,             "insert admin" },
+    { U_V1902_DROP_TABLE_ADMINTMP,      "drop table admin_tmp" },
+
+    { U_V1902_SCVER_MAJOR,    "set schema_version_major to 19" },
+    { U_V1902_SCVER_MINOR,    "set schema_version_minor to 02" },
+  };
+
+
+#define U_V1903_ALTER_QUEUE_ADD_ARTWORKURL \
+  "ALTER TABLE queue ADD COLUMN artwork_url VARCHAR(4096) DEFAULT NULL;"
+
+#define U_V1903_SCVER_MAJOR \
+  "UPDATE admin SET value = '19' WHERE key = 'schema_version_major';"
+#define U_V1903_SCVER_MINOR \
+  "UPDATE admin SET value = '03' WHERE key = 'schema_version_minor';"
+
+static const struct db_upgrade_query db_upgrade_v1903_queries[] =
+  {
+    { U_V1903_ALTER_QUEUE_ADD_ARTWORKURL,    "alter table queue add column artwork_url" },
+
+    { U_V1903_SCVER_MAJOR,    "set schema_version_major to 19" },
+    { U_V1903_SCVER_MINOR,    "set schema_version_minor to 03" },
+  };
+
+
+#define U_V1904_ALTER_SPEAKERS_ADD_AUTHKEY \
+  "ALTER TABLE speakers ADD COLUMN auth_key VARCHAR(2048) DEFAULT NULL;"
+
+#define U_V1904_SCVER_MAJOR \
+  "UPDATE admin SET value = '19' WHERE key = 'schema_version_major';"
+#define U_V1904_SCVER_MINOR \
+  "UPDATE admin SET value = '04' WHERE key = 'schema_version_minor';"
+
+static const struct db_upgrade_query db_upgrade_v1904_queries[] =
+  {
+    { U_V1904_ALTER_SPEAKERS_ADD_AUTHKEY,    "alter table speakers add column auth_key" },
+
+    { U_V1904_SCVER_MAJOR,    "set schema_version_major to 19" },
+    { U_V1904_SCVER_MINOR,    "set schema_version_minor to 04" },
+  };
+
+
 int
 db_upgrade(sqlite3 *hdl, int db_ver)
 {
@@ -1552,6 +1674,34 @@ db_upgrade(sqlite3 *hdl, int db_ver)
       if (ret < 0)
 	return -1;
 
+      /* FALLTHROUGH */
+
+    case 1900:
+      ret = db_generic_upgrade(hdl, db_upgrade_v1901_queries, sizeof(db_upgrade_v1901_queries) / sizeof(db_upgrade_v1901_queries[0]));
+      if (ret < 0)
+	return -1;
+
+      /* FALLTHROUGH */
+
+    case 1901:
+      ret = db_generic_upgrade(hdl, db_upgrade_v1902_queries, sizeof(db_upgrade_v1902_queries) / sizeof(db_upgrade_v1902_queries[0]));
+      if (ret < 0)
+	return -1;
+
+      /* FALLTHROUGH */
+
+    case 1902:
+      ret = db_generic_upgrade(hdl, db_upgrade_v1903_queries, sizeof(db_upgrade_v1903_queries) / sizeof(db_upgrade_v1903_queries[0]));
+      if (ret < 0)
+	return -1;
+
+      /* FALLTHROUGH */
+
+    case 1903:
+      ret = db_generic_upgrade(hdl, db_upgrade_v1904_queries, sizeof(db_upgrade_v1904_queries) / sizeof(db_upgrade_v1904_queries[0]));
+      if (ret < 0)
+	return -1;
+
       break;
 
     default:
