@@ -2,6 +2,7 @@
 #ifndef __DMAP_HELPERS_H__
 #define __DMAP_HELPERS_H__
 
+#include "config.h"
 #include <event2/buffer.h>
 #include <event2/http.h>
 
@@ -46,7 +47,7 @@ dmap_get_fields_table(int *nfields);
 
 /* From dmap_fields.gperf - keep in sync, don't alter */
 const struct dmap_field *
-dmap_find_field (register const char *str, register unsigned int len);
+dmap_find_field (register const char *str, register GPERF_LEN_TYPE len);
 
 
 void
@@ -84,4 +85,7 @@ dmap_send_error(struct evhttp_request *req, const char *container, const char *e
 int
 dmap_encode_file_metadata(struct evbuffer *songlist, struct evbuffer *song, struct db_media_file_info *dbmfi, const struct dmap_field **meta, int nmeta, int sort_tags, int force_wav);
 
+int
+dmap_encode_queue_metadata(struct evbuffer *songlist, struct evbuffer *song, struct db_queue_item *queue_item);
+
 #endif /* !__DMAP_HELPERS_H__ */
