@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Espen Jürgensen <espenjurgensen@gmail.com>
+ * Copyright (C) 2016 Espen Jürgensen <espenjurgensen@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,10 @@
 
 #include <event2/event.h>
 
+#ifdef HAVE_LIBCURL
+#include <curl/curl.h>
+#endif
+
 #include "http.h"
 #include "logger.h"
 #include "misc.h"
@@ -128,10 +132,10 @@ request_cb(struct evhttp_request *req, void *arg)
 
   ctx->ret = 0;
 
-  if (ctx->headers)
-    headers_save(ctx->headers, evhttp_request_get_input_headers(req));
-  if (ctx->body)
-    evbuffer_add_buffer(ctx->body, evhttp_request_get_input_buffer(req));
+  if (ctx->input_headers)
+    headers_save(ctx->input_headers, evhttp_request_get_input_headers(req));
+  if (ctx->input_body)
+    evbuffer_add_buffer(ctx->input_body, evhttp_request_get_input_buffer(req));
       
   event_base_loopbreak(ctx->evbase);
 
@@ -159,34 +163,38 @@ request_header_cb(struct evhttp_request *req, void *arg)
 
   ctx = (struct http_client_ctx *)arg;
 
-  if (!ctx->headers)
+  if (!ctx->input_headers)
     {
       DPRINTF(E_LOG, L_HTTP, "BUG: Header callback invoked but caller did not say where to save the headers\n");
       return -1;
     }
 
-  headers_save(ctx->headers, evhttp_request_get_input_headers(req));
+  headers_save(ctx->input_headers, evhttp_request_get_input_headers(req));
 
   return -1;
 }
 #endif
 
-int
-http_client_request(struct http_client_ctx *ctx)
+static int
+http_client_request_impl(struct http_client_ctx *ctx)
 {
   struct evhttp_connection *evcon;
   struct evhttp_request *req;
   struct evkeyvalq *headers;
-  char hostname[PATH_MAX];
-  char path[PATH_MAX];
-  char s[PATH_MAX];
+  struct evbuffer *output_buffer;
+  struct onekeyval *okv;
+  enum evhttp_cmd_type method;
+  char host[512];
+  char host_port[1024];
+  char path[2048];
+  char tmp[128];
   int port;
   int ret;
 
   ctx->ret = -1;
 
-  av_url_split(NULL, 0, NULL, 0, hostname, sizeof(hostname), &port, path, sizeof(path), ctx->url);
-  if (strlen(hostname) == 0)
+  av_url_split(NULL, 0, NULL, 0, host, sizeof(host), &port, path, sizeof(path), ctx->url);
+  if (strlen(host) == 0)
     {
       DPRINTF(E_LOG, L_HTTP, "Error extracting hostname from URL: %s\n", ctx->url);
 
@@ -194,9 +202,9 @@ http_client_request(struct http_client_ctx *ctx)
     }
 
   if (port <= 0)
-    snprintf(s, PATH_MAX, "%s", hostname);
+    snprintf(host_port, sizeof(host_port), "%s", host);
   else
-    snprintf(s, PATH_MAX, "%s:%d", hostname, port);
+    snprintf(host_port, sizeof(host_port), "%s:%d", host, port);
 
   if (port <= 0)
     port = 80;
@@ -215,10 +223,10 @@ http_client_request(struct http_client_ctx *ctx)
       return ctx->ret;
     }
 
-  evcon = evhttp_connection_base_new(ctx->evbase, NULL, hostname, (unsigned short)port);
+  evcon = evhttp_connection_base_new(ctx->evbase, NULL, host, (unsigned short)port);
   if (!evcon)
     {
-      DPRINTF(E_LOG, L_HTTP, "Could not create connection to %s\n", hostname);
+      DPRINTF(E_LOG, L_HTTP, "Could not create connection to %s\n", host_port);
 
       event_base_free(ctx->evbase);
       return ctx->ret;
@@ -230,7 +238,7 @@ http_client_request(struct http_client_ctx *ctx)
   req = evhttp_request_new(request_cb, ctx);
   if (!req)
     {
-      DPRINTF(E_LOG, L_HTTP, "Could not create request to %s\n", hostname);
+      DPRINTF(E_LOG, L_HTTP, "Could not create request to %s\n", host_port);
 
       evhttp_connection_free(evcon);
       event_base_free(ctx->evbase);
@@ -243,18 +251,38 @@ http_client_request(struct http_client_ctx *ctx)
 #endif
 
   headers = evhttp_request_get_output_headers(req);
-  evhttp_add_header(headers, "Host", s);
-  evhttp_add_header(headers, "Content-Length", "0");
+  evhttp_add_header(headers, "Host", host_port);
   evhttp_add_header(headers, "User-Agent", "forked-daapd/" VERSION);
   evhttp_add_header(headers, "Icy-MetaData", "1");
 
-  /* Make request */
-  DPRINTF(E_INFO, L_HTTP, "Making request for http://%s%s\n", s, path);
+  if (ctx->output_headers)
+    {
+      for (okv = ctx->output_headers->head; okv; okv = okv->next)
+	evhttp_add_header(headers, okv->name, okv->value);
+    }
 
-  ret = evhttp_make_request(evcon, req, EVHTTP_REQ_GET, path);
+  if (ctx->output_body)
+    {
+      output_buffer = evhttp_request_get_output_buffer(req);
+      evbuffer_add(output_buffer, ctx->output_body, strlen(ctx->output_body));
+      evbuffer_add_printf(output_buffer, "\n");
+      snprintf(tmp, sizeof(tmp), "%zu", evbuffer_get_length(output_buffer));
+      evhttp_add_header(headers, "Content-Length", tmp);
+      method = EVHTTP_REQ_POST;
+    }
+  else
+    {
+      evhttp_add_header(headers, "Content-Length", "0");
+      method = EVHTTP_REQ_GET;
+    }
+
+  /* Make request */
+  DPRINTF(E_INFO, L_HTTP, "Making %s request for http://%s%s\n", ((method==EVHTTP_REQ_GET) ? "GET" : "POST"), host_port, path);
+
+  ret = evhttp_make_request(evcon, req, method, path);
   if (ret < 0)
     {
-      DPRINTF(E_LOG, L_HTTP, "Error making request for http://%s%s\n", s, path);
+      DPRINTF(E_LOG, L_HTTP, "Error making request for http://%s%s\n", host_port, path);
 
       evhttp_connection_free(evcon);
       event_base_free(ctx->evbase);
@@ -269,6 +297,144 @@ http_client_request(struct http_client_ctx *ctx)
   return ctx->ret;
 }
 
+#ifdef HAVE_LIBCURL
+static size_t
+curl_request_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
+{
+  size_t realsize;
+  struct http_client_ctx *ctx;
+  int ret;
+
+  realsize = size * nmemb;
+  ctx = (struct http_client_ctx *)userdata;
+
+  if (!ctx->input_body)
+    return realsize;
+
+  ret = evbuffer_add(ctx->input_body, ptr, realsize);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_HTTP, "Error adding reply from %s to input buffer\n", ctx->url);
+      return 0;
+    }
+
+  return realsize;
+}
+
+static int
+https_client_request_impl(struct http_client_ctx *ctx)
+{
+  CURL *curl;
+  CURLcode res;
+  struct curl_slist *headers;
+  struct onekeyval *okv;
+  char header[1024];
+
+  curl = curl_easy_init();
+  if (!curl)
+    {
+      DPRINTF(E_LOG, L_HTTP, "Error: Could not get curl handle\n");
+      return -1;
+    }
+
+  curl_easy_setopt(curl, CURLOPT_URL, ctx->url);
+  curl_easy_setopt(curl, CURLOPT_USERAGENT, "forked-daapd/" VERSION);
+
+  if (ctx->output_headers)
+    {
+      headers = NULL;
+      for (okv = ctx->output_headers->head; okv; okv = okv->next)
+	{
+	  snprintf(header, sizeof(header), "%s: %s", okv->name, okv->value);
+	  headers = curl_slist_append(headers, header);
+        }
+
+      curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
+    }
+
+  if (ctx->output_body)
+    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, ctx->output_body);
+
+  curl_easy_setopt(curl, CURLOPT_TIMEOUT, HTTP_CLIENT_TIMEOUT);
+
+  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_request_cb);
+  curl_easy_setopt(curl, CURLOPT_WRITEDATA, ctx);
+
+  /* Make request */
+  DPRINTF(E_INFO, L_HTTP, "Making request for %s\n", ctx->url);
+
+  res = curl_easy_perform(curl);
+  if (res != CURLE_OK)
+    {
+      DPRINTF(E_LOG, L_HTTP, "Request to %s failed: %s\n", ctx->url, curl_easy_strerror(res));
+      curl_easy_cleanup(curl);
+      return -1;
+    }
+
+  curl_easy_cleanup(curl);
+
+  return 0;
+}
+#endif /* HAVE_LIBCURL */
+
+int
+http_client_request(struct http_client_ctx *ctx)
+{
+  if (strncmp(ctx->url, "http:", strlen("http:")) == 0)
+    return http_client_request_impl(ctx);
+
+#ifdef HAVE_LIBCURL
+  if (strncmp(ctx->url, "https:", strlen("https:")) == 0)
+    return https_client_request_impl(ctx);
+#endif
+
+  DPRINTF(E_LOG, L_HTTP, "Request for %s is not supported (not built with libcurl?)\n", ctx->url);
+  return -1;
+}
+
+char *
+http_form_urlencode(struct keyval *kv)
+{
+  struct evbuffer *evbuf;
+  struct onekeyval *okv;
+  char *body;
+  char *k;
+  char *v;
+
+  evbuf = evbuffer_new();
+
+  for (okv = kv->head; okv; okv = okv->next)
+    {
+      k = evhttp_encode_uri(okv->name);
+      if (!k)
+        continue;
+
+      v = evhttp_encode_uri(okv->value);
+      if (!v)
+	{
+	  free(k);
+	  continue;
+	}
+
+      evbuffer_add_printf(evbuf, "%s=%s", k, v);
+      if (okv->next)
+	evbuffer_add_printf(evbuf, "&");
+
+      free(k);
+      free(v);
+    }
+
+  evbuffer_add(evbuf, "\n", 1);
+
+  body = evbuffer_readln(evbuf, NULL, EVBUFFER_EOL_ANY);
+
+  evbuffer_free(evbuf);
+
+  DPRINTF(E_DBG, L_HTTP, "Parameters in request are: %s\n", body);
+
+  return body;
+}
+
 int
 http_stream_setup(char **stream, const char *url)
 {
@@ -282,7 +448,7 @@ http_stream_setup(char **stream, const char *url)
   *stream = NULL;
 
   ext = strrchr(url, '.');
-  if (strcasecmp(ext, ".m3u") != 0)
+  if (!ext || (strcasecmp(ext, ".m3u") != 0))
     {
       *stream = strdup(url);
       return 0;
@@ -296,7 +462,7 @@ http_stream_setup(char **stream, const char *url)
     return -1;
 
   ctx.url = url;
-  ctx.body = evbuf;
+  ctx.input_body = evbuf;
 
   ret = http_client_request(&ctx);
   if (ret < 0)
@@ -308,13 +474,13 @@ http_stream_setup(char **stream, const char *url)
     }
 
   // Pad with CRLF because evbuffer_readln() might not read the last line otherwise
-  evbuffer_add(ctx.body, "\r\n", 2);
+  evbuffer_add(ctx.input_body, "\r\n", 2);
 
   /* Read the playlist until the first stream link is found, but give up if
    * nothing is found in the first 10 lines
    */
   n = 0;
-  while ((line = evbuffer_readln(ctx.body, NULL, EVBUFFER_EOL_ANY)) && (n < 10))
+  while ((line = evbuffer_readln(ctx.input_body, NULL, EVBUFFER_EOL_ANY)) && (n < 10))
     {
       n++;
       if (strncasecmp(line, "http://", strlen("http://")) == 0)
@@ -328,7 +494,7 @@ http_stream_setup(char **stream, const char *url)
       free(line);
     }
 
-  evbuffer_free(ctx.body);
+  evbuffer_free(ctx.input_body);
 
   if (n != -1)
     {
@@ -529,9 +695,9 @@ http_icy_metadata_get(AVFormatContext *fmtctx, int packet_only)
 
   memset(&ctx, 0, sizeof(struct http_client_ctx));
   ctx.url = fmtctx->filename;
-  ctx.headers = kv;
+  ctx.input_headers = kv;
   ctx.headers_only = 1;
-  ctx.body = NULL;
+  ctx.input_body = NULL;
 
   ret = http_client_request(&ctx);
   if (ret < 0)
@@ -548,17 +714,17 @@ http_icy_metadata_get(AVFormatContext *fmtctx, int packet_only)
   memset(metadata, 0, sizeof(struct http_icy_metadata));
 
   got_header = 0;
-  if ( (value = keyval_get(ctx.headers, "icy-name")) )
+  if ( (value = keyval_get(ctx.input_headers, "icy-name")) )
     {
       metadata->name = strdup(value);
       got_header = 1;
     }
-  if ( (value = keyval_get(ctx.headers, "icy-description")) )
+  if ( (value = keyval_get(ctx.input_headers, "icy-description")) )
     {
       metadata->description = strdup(value);
       got_header = 1;
     }
-  if ( (value = keyval_get(ctx.headers, "icy-genre")) )
+  if ( (value = keyval_get(ctx.input_headers, "icy-genre")) )
     {
       metadata->genre = strdup(value);
       got_header = 1;
