@@ -39,11 +39,11 @@
 #include <stdint.h>
 #include <inttypes.h>
 
-#if defined(HAVE_SYS_EVENTFD_H) && defined(HAVE_EVENTFD)
-# define USE_EVENTFD
+#ifdef HAVE_EVENTFD
 # include <sys/eventfd.h>
 #endif
 #include <event2/event.h>
+#include <event2/keyvalq_struct.h>
 #ifdef HAVE_LIBEVENT2_OLD
 # include <event2/bufferevent.h>
 # include <event2/bufferevent_struct.h>
@@ -64,6 +64,9 @@
 #ifdef LASTFM
 # include "lastfm.h"
 #endif
+#ifdef HAVE_SPOTIFY_H
+# include "spotify.h"
+#endif
 
 /*
  * HTTP client quirks by User-Agent, from mt-daapd
@@ -129,7 +132,7 @@ static const struct content_type_map ext2ctype[] =
 
 struct event_base *evbase_httpd;
 
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
 static int exit_efd;
 #else
 static int exit_pipe[2];
@@ -139,7 +142,8 @@ static struct event *exitev;
 static struct evhttp *evhttpd;
 static pthread_t tid_httpd;
 
-static char *allow_origin;
+static const char *allow_origin;
+static int httpd_port;
 
 #ifdef HAVE_LIBEVENT2_OLD
 struct stream_ctx *g_st;
@@ -198,6 +202,63 @@ scrobble_cb(void *arg)
 }
 #endif
 
+static void
+oauth_interface(struct evhttp_request *req, const char *uri)
+{
+  struct evbuffer *evbuf;
+  struct evkeyvalq query;
+  const char *req_uri;
+  const char *ptr;
+  char __attribute__((unused)) redirect_uri[256];
+  int ret;
+
+  req_uri = evhttp_request_get_uri(req);
+
+  evbuf = evbuffer_new();
+  if (!evbuf)
+    {
+      DPRINTF(E_LOG, L_HTTPD, "Could not alloc evbuf for oauth\n");
+      return;
+    }
+
+  evbuffer_add_printf(evbuf, "<H1>forked-daapd oauth</H1>\n\n");
+
+  memset(&query, 0, sizeof(struct evkeyvalq));
+
+  ptr = strchr(req_uri, '?');
+  if (ptr)
+    {
+      ret = evhttp_parse_query_str(ptr + 1, &query);
+      if (ret < 0)
+	{
+	  evbuffer_add_printf(evbuf, "OAuth error: Could not parse parameters in callback (%s)\n", req_uri);
+
+	  httpd_send_reply(req, HTTP_OK, "OK", evbuf, 0);
+	  evbuffer_free(evbuf);
+	  return;
+	}
+    }
+
+#ifdef HAVE_SPOTIFY_H
+  snprintf(redirect_uri, sizeof(redirect_uri), "http://forked-daapd.local:%d/oauth/spotify", httpd_port);
+
+  if (strncmp(uri, "/oauth/spotify", strlen("/oauth/spotify")) == 0)
+    spotify_oauth_callback(evbuf, &query, redirect_uri);
+  else
+    spotify_oauth_interface(evbuf, redirect_uri);
+#else
+  evbuffer_add_printf(evbuf, "<p>This version was built without modules requiring OAuth support</p>\n");
+#endif
+
+  evbuffer_add_printf(evbuf, "<p><i>(sorry about this ugly interface)</i></p>\n");
+
+  evhttp_clear_headers(&query);
+
+  httpd_send_reply(req, HTTP_OK, "OK", evbuf, 0);
+
+  evbuffer_free(evbuf);
+}
+
 static void
 stream_end_register(struct stream_ctx *st)
 {
@@ -491,7 +552,7 @@ httpd_stream_file(struct evhttp_request *req, int id)
 
       stream_cb = stream_chunk_xcode_cb;
 
-      st->xcode = transcode_setup(mfi, XCODE_PCM16_HEADER, &st->size);
+      st->xcode = transcode_setup(mfi->data_kind, mfi->path, mfi->song_length, XCODE_PCM16_HEADER, &st->size);
       if (!st->xcode)
 	{
 	  DPRINTF(E_WARN, L_HTTPD, "Transcoding setup failed, aborting streaming\n");
@@ -875,11 +936,11 @@ static void
 serve_file(struct evhttp_request *req, char *uri)
 {
   const char *host;
+  const char *passwd;
   char *ext;
   char path[PATH_MAX];
   char *deref;
   char *ctype;
-  char *passwd;
   struct evbuffer *evbuf;
   struct evkeyvalq *headers;
   struct stat sb;
@@ -912,6 +973,12 @@ serve_file(struct evhttp_request *req, char *uri)
 	}
     }
 
+  if (strncmp(uri, "/oauth", strlen("/oauth")) == 0)
+    {
+      oauth_interface(req, uri);
+      return;
+    }
+
   ret = snprintf(path, sizeof(path), "%s%s", WEBFACE_ROOT, uri + 1); /* skip starting '/' */
   if ((ret < 0) || (ret >= sizeof(path)))
     {
@@ -1244,15 +1311,14 @@ httpd_fixup_uri(struct evhttp_request *req)
 static const char *http_reply_401 = "<html><head><title>401 Unauthorized</title></head><body>Authorization required</body></html>";
 
 int
-httpd_basic_auth(struct evhttp_request *req, char *user, char *passwd, char *realm)
+httpd_basic_auth(struct evhttp_request *req, const char *user, const char *passwd, const char *realm)
 {
   struct evbuffer *evbuf;
   struct evkeyvalq *headers;
-  char *header;
+  char header[256];
   const char *auth;
   char *authuser;
   char *authpwd;
-  int len;
   int ret;
 
   headers = evhttp_request_get_input_headers(req);
@@ -1317,16 +1383,8 @@ httpd_basic_auth(struct evhttp_request *req, char *user, char *passwd, char *rea
   return 0;
 
  need_auth:
-  len = strlen(realm) + strlen("Basic realm=") + 3;
-  header = (char *)malloc(len);
-  if (!header)
-    {
-      httpd_send_error(req, HTTP_SERVUNAVAIL, "Internal Server Error");
-      return -1;
-    }
-
-  ret = snprintf(header, len, "Basic realm=\"%s\"", realm);
-  if ((ret < 0) || (ret >= len))
+  ret = snprintf(header, sizeof(header), "Basic realm=\"%s\"", realm);
+  if ((ret < 0) || (ret >= sizeof(header)))
     {
       httpd_send_error(req, HTTP_SERVUNAVAIL, "Internal Server Error");
       return -1;
@@ -1346,7 +1404,6 @@ httpd_basic_auth(struct evhttp_request *req, char *user, char *passwd, char *rea
 
   httpd_send_reply(req, 401, "Unauthorized", evbuf, HTTPD_SEND_NO_GZIP);
 
-  free(header);
   evbuffer_free(evbuf);
 
   return -1;
@@ -1357,7 +1414,6 @@ int
 httpd_init(void)
 {
   int v6enabled;
-  unsigned short port;
   int ret;
 
   httpd_exit = 0;
@@ -1396,7 +1452,7 @@ httpd_init(void)
 
   streaming_init();
 
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   exit_efd = eventfd(0, EFD_CLOEXEC);
   if (exit_efd < 0)
     {
@@ -1420,7 +1476,7 @@ httpd_init(void)
     }
 
   exitev = event_new(evbase_httpd, exit_pipe[0], EV_READ, exit_cb, NULL);
-#endif /* USE_EVENTFD */
+#endif /* HAVE_EVENTFD */
   if (!exitev)
     {
       DPRINTF(E_FATAL, L_HTTPD, "Could not create exit event\n");
@@ -1438,7 +1494,7 @@ httpd_init(void)
     }
 
   v6enabled = cfg_getbool(cfg_getsec(cfg, "general"), "ipv6");
-  port = cfg_getint(cfg_getsec(cfg, "library"), "port");
+  httpd_port = cfg_getint(cfg_getsec(cfg, "library"), "port");
 
   // For CORS headers
   allow_origin = cfg_getstr(cfg_getsec(cfg, "general"), "allow_origin");
@@ -1452,20 +1508,20 @@ httpd_init(void)
 
   if (v6enabled)
     {
-      ret = evhttp_bind_socket(evhttpd, "::", port);
+      ret = evhttp_bind_socket(evhttpd, "::", httpd_port);
       if (ret < 0)
 	{
-	  DPRINTF(E_LOG, L_HTTPD, "Could not bind to port %d with IPv6, falling back to IPv4\n", port);
+	  DPRINTF(E_LOG, L_HTTPD, "Could not bind to port %d with IPv6, falling back to IPv4\n", httpd_port);
 	  v6enabled = 0;
 	}
     }
 
   if (!v6enabled)
     {
-      ret = evhttp_bind_socket(evhttpd, "0.0.0.0", port);
+      ret = evhttp_bind_socket(evhttpd, "0.0.0.0", httpd_port);
       if (ret < 0)
 	{
-	  DPRINTF(E_FATAL, L_HTTPD, "Could not bind to port %d (forked-daapd already running?)\n", port);
+	  DPRINTF(E_FATAL, L_HTTPD, "Could not bind to port %d (forked-daapd already running?)\n", httpd_port);
 	  goto bind_fail;
 	}
     }
@@ -1492,7 +1548,7 @@ httpd_init(void)
  bind_fail:
   evhttp_free(evhttpd);
  event_fail:
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   close(exit_efd);
 #else
   close(exit_pipe[0]);
@@ -1517,7 +1573,7 @@ httpd_deinit(void)
 {
   int ret;
 
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   ret = eventfd_write(exit_efd, 1);
   if (ret < 0)
     {
@@ -1550,7 +1606,7 @@ httpd_deinit(void)
   dacp_deinit();
   daap_deinit();
 
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   close(exit_efd);
 #else
   close(exit_pipe[0]);
