@@ -2,8 +2,8 @@
 #ifndef __LASTFM_H__
 #define __LASTFM_H__
 
-int
-lastfm_login(char *path);
+void
+lastfm_login(char **arglist);
 
 int
 lastfm_scrobble(int id);
