@@ -37,21 +37,23 @@
 #include <grp.h>
 #include <stdint.h>
 
-#if defined(__linux__)
+#ifdef HAVE_SIGNALFD
 # include <sys/signalfd.h>
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#else
 # include <sys/time.h>
 # include <sys/event.h>
 #endif
 
-#include <pthread.h>
-
 #include <getopt.h>
 #include <event2/event.h>
+#ifdef HAVE_LIBEVENT_PTHREADS
+# include <event2/thread.h>
+#endif
 #include <libavutil/log.h>
 #include <libavformat/avformat.h>
 #include <libavfilter/avfilter.h>
 
+#include <pthread.h>
 #include <gcrypt.h>
 GCRY_THREAD_OPTION_PTHREAD_IMPL;
 
@@ -60,20 +62,17 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include "logger.h"
 #include "misc.h"
 #include "cache.h"
-#include "filescanner.h"
 #include "httpd.h"
 #include "mpd.h"
 #include "mdns.h"
 #include "remote_pairing.h"
 #include "player.h"
 #include "worker.h"
+#include "library.h"
 
-#ifdef LASTFM
+#ifdef HAVE_LIBCURL
 # include <curl/curl.h>
 #endif
-#ifdef HAVE_SPOTIFY_H
-# include "spotify.h"
-#endif
 
 #define PIDFILE   STATEDIR "/run/" PACKAGE ".pid"
 
@@ -334,7 +333,7 @@ register_services(char *ffid, int no_rsp, int no_daap, int mdns_no_mpd)
 }
 
 
-#if defined(__linux__)
+#ifdef HAVE_SIGNALFD
 static void
 signal_signalfd_cb(int fd, short event, void *arg)
 {
@@ -374,7 +373,7 @@ signal_signalfd_cb(int fd, short event, void *arg)
     event_add(sig_event, NULL);
 }
 
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#else
 
 static void
 signal_kqueue_cb(int fd, short event, void *arg)
@@ -422,26 +421,29 @@ signal_kqueue_cb(int fd, short event, void *arg)
 
 
 static int
-ffmpeg_lockmgr(void **mutex, enum AVLockOp op)
+ffmpeg_lockmgr(void **pmutex, enum AVLockOp op)
 {
   switch (op)
     {
       case AV_LOCK_CREATE:
-	*mutex = malloc(sizeof(pthread_mutex_t));
-	if (!*mutex)
+	*pmutex = malloc(sizeof(pthread_mutex_t));
+	if (!*pmutex)
 	  return 1;
-
-	return !!pthread_mutex_init(*mutex, NULL);
+        CHECK_ERR(L_MAIN, mutex_init(*pmutex));
+	return 0;
 
       case AV_LOCK_OBTAIN:
-	return !!pthread_mutex_lock(*mutex);
+        CHECK_ERR(L_MAIN, pthread_mutex_lock(*pmutex));
+	return 0;
 
       case AV_LOCK_RELEASE:
-	return !!pthread_mutex_unlock(*mutex);
+        CHECK_ERR(L_MAIN, pthread_mutex_unlock(*pmutex));
+	return 0;
 
       case AV_LOCK_DESTROY:
-	pthread_mutex_destroy(*mutex);
-	free(*mutex);
+	CHECK_ERR(L_MAIN, pthread_mutex_destroy(*pmutex));
+	free(*pmutex);
+        *pmutex = NULL;
 
 	return 0;
     }
@@ -449,7 +451,6 @@ ffmpeg_lockmgr(void **mutex, enum AVLockOp op)
   return 1;
 }
 
-
 int
 main(int argc, char **argv)
 {
@@ -468,7 +469,7 @@ main(int argc, char **argv)
   const char *gcry_version;
   sigset_t sigs;
   int sigfd;
-#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#ifdef HAVE_KQUEUE
   struct kevent ke_sigs[4];
 #endif
   int ret;
@@ -498,7 +499,6 @@ main(int argc, char **argv)
   ffid = NULL;
   mdns_no_rsp = 0;
   mdns_no_daap = 0;
-  mdns_no_mpd = 1; // only announce if mpd protocol support is activated
 
   while ((option = getopt_long(argc, argv, "D:d:c:P:fb:v", option_map, NULL)) != -1)
     {
@@ -608,14 +608,17 @@ main(int argc, char **argv)
 #ifdef MPD
   strcat(buildopts, " --enable-mpd");
 #endif
-#ifdef ALSA
+#ifdef RAOP_VERIFICATION
+  strcat(buildopts, " --enable-verification");
+#endif
+#ifdef HAVE_ALSA
   strcat(buildopts, " --with-alsa");
 #endif
-#ifdef PULSEAUDIO
+#ifdef HAVE_LIBPULSE
   strcat(buildopts, " --with-pulseaudio");
 #endif
 
-  DPRINTF(E_LOG, L_MAIN, "Built %s with:%s\n", BUILDDATE, buildopts);
+  DPRINTF(E_LOG, L_MAIN, "Built %s with:%s\n", __DATE__, buildopts);
 
   ret = av_lockmgr_register(ffmpeg_lockmgr);
   if (ret < 0)
@@ -633,7 +636,7 @@ main(int argc, char **argv)
 #endif
   av_log_set_callback(logger_ffmpeg);
 
-#ifdef LASTFM
+#ifdef HAVE_LIBCURL
   /* Initialize libcurl */
   curl_global_init(CURL_GLOBAL_DEFAULT);
 #endif
@@ -686,7 +689,11 @@ main(int argc, char **argv)
     }
 
   /* Initialize event base (after forking) */
-  evbase_main = event_base_new();
+  CHECK_NULL(L_MAIN, evbase_main = event_base_new());
+
+#ifdef HAVE_LIBEVENT_PTHREADS
+  CHECK_ERR(L_MAIN, evthread_use_pthreads());
+#endif
 
   DPRINTF(E_LOG, L_MAIN, "mDNS init\n");
   ret = mdns_init();
@@ -739,25 +746,16 @@ main(int argc, char **argv)
       goto cache_fail;
     }
 
-  /* Spawn file scanner thread */
-  ret = filescanner_init();
+  /* Spawn library scan thread */
+  ret = library_init();
   if (ret != 0)
     {
-      DPRINTF(E_FATAL, L_MAIN, "File scanner thread failed to start\n");
+      DPRINTF(E_FATAL, L_MAIN, "Library thread failed to start\n");
 
       ret = EXIT_FAILURE;
-      goto filescanner_fail;
+      goto library_fail;
     }
 
-#ifdef HAVE_SPOTIFY_H
-  /* Spawn Spotify thread */
-  ret = spotify_init();
-  if (ret < 0)
-    {
-      DPRINTF(E_INFO, L_MAIN, "Spotify thread not started\n");;
-    }
-#endif
-
   /* Spawn player thread */
   ret = player_init();
   if (ret != 0)
@@ -789,6 +787,8 @@ main(int argc, char **argv)
       goto mpd_fail;
     }
   mdns_no_mpd = 0;
+#else
+  mdns_no_mpd = 1;
 #endif
 
   /* Start Remote pairing service */
@@ -809,7 +809,10 @@ main(int argc, char **argv)
       goto mdns_reg_fail;
     }
 
-#if defined(__linux__)
+  /* Register this CNAME with mDNS for OAuth */
+  mdns_cname("forked-daapd.local");
+
+#ifdef HAVE_SIGNALFD
   /* Set up signal fd */
   sigfd = signalfd(-1, &sigs, SFD_NONBLOCK | SFD_CLOEXEC);
   if (sigfd < 0)
@@ -821,7 +824,7 @@ main(int argc, char **argv)
     }
 
   sig_event = event_new(evbase_main, sigfd, EV_READ, signal_signalfd_cb, NULL);
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#else
   sigfd = kqueue();
   if (sigfd < 0)
     {
@@ -891,14 +894,10 @@ main(int argc, char **argv)
   player_deinit();
 
  player_fail:
-#ifdef HAVE_SPOTIFY_H
-  DPRINTF(E_LOG, L_MAIN, "Spotify deinit\n");
-  spotify_deinit();
-#endif
-  DPRINTF(E_LOG, L_MAIN, "File scanner deinit\n");
-  filescanner_deinit();
+  DPRINTF(E_LOG, L_MAIN, "Library scaner deinit\n");
+  library_deinit();
 
- filescanner_fail:
+ library_fail:
   DPRINTF(E_LOG, L_MAIN, "Cache deinit\n");
   cache_deinit();
 
@@ -935,7 +934,7 @@ main(int argc, char **argv)
 
  signal_block_fail:
  gcrypt_init_fail:
-#ifdef LASTFM
+#ifdef HAVE_LIBCURL
   curl_global_cleanup();
 #endif
 #if LIBAVFORMAT_VERSION_MAJOR >= 54 || (LIBAVFORMAT_VERSION_MAJOR == 53 && LIBAVFORMAT_VERSION_MINOR >= 13)
