@@ -33,6 +33,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <net/if.h>
+#include <unistd.h>
 
 #include <event2/event.h>
 
@@ -43,6 +44,12 @@
 #include <avahi-client/publish.h>
 #include <avahi-client/lookup.h>
 
+// Hack for FreeBSD, don't want to bother with sysconf()
+#ifndef HOST_NAME_MAX
+# include <limits.h>
+# define HOST_NAME_MAX _POSIX_HOST_NAME_MAX
+#endif
+
 #include "logger.h"
 #include "mdns.h"
 
@@ -351,8 +358,24 @@ struct mdns_record_browser {
   int port;
 };
 
+struct mdns_resolver
+{
+  char *name;
+  AvahiServiceResolver *resolver;
+  AvahiProtocol proto;
+
+  struct mdns_resolver *next;
+};
+
+enum publish
+{
+  MDNS_PUBLISH_SERVICE,
+  MDNS_PUBLISH_CNAME,
+};
+
 struct mdns_group_entry
 {
+  enum publish publish;
   char *name;
   char *type;
   int port;
@@ -362,6 +385,7 @@ struct mdns_group_entry
 };
 
 static struct mdns_browser *browser_list;
+static struct mdns_resolver *resolver_list;
 static struct mdns_group_entry *group_entries;
 
 #define IPV4LL_NETWORK 0xA9FE0000
@@ -416,6 +440,36 @@ avahi_address_make(AvahiAddress *addr, AvahiProtocol proto, const void *rdata, s
   return -1;
 }
 
+// Frees all resolvers for a given service name
+static void
+resolvers_cleanup(const char *name, AvahiProtocol proto)
+{
+  struct mdns_resolver *r;
+  struct mdns_resolver *prev;
+  struct mdns_resolver *next;
+
+  prev = NULL;
+  for (r = resolver_list; r; r = next)
+    {
+      next = r->next;
+
+      if ((strcmp(name, r->name) != 0) || (proto != r->proto))
+	{
+	  prev = r;
+	  continue;
+	}
+
+      if (!prev)
+	resolver_list = r->next;
+      else
+	prev->next = r->next;
+
+      avahi_service_resolver_free(r->resolver);
+      free(r->name);
+      free(r);
+    }
+}
+
 static void
 browse_record_callback(AvahiRecordBrowser *b, AvahiIfIndex intf, AvahiProtocol proto,
                        AvahiBrowserEvent event, const char *hostname, uint16_t clazz, uint16_t type,
@@ -478,33 +532,40 @@ browse_resolve_callback(AvahiServiceResolver *r, AvahiIfIndex intf, AvahiProtoco
 			uint16_t port, AvahiStringList *txt, AvahiLookupResultFlags flags, void *userdata)
 {
   AvahiRecordBrowser *rb;
+  struct mdns_browser *mb;
   struct mdns_record_browser *rb_data;
   char *key;
   char *value;
   uint16_t dns_type;
+  int family;
   int ret;
 
-  if (event == AVAHI_RESOLVER_FAILURE)
+  mb = (struct mdns_browser *)userdata;
+
+  if (event != AVAHI_RESOLVER_FOUND)
     {
-      DPRINTF(E_LOG, L_MDNS, "Avahi Resolver failure: service '%s' type '%s': %s\n", name, type, MDNSERR);
-      goto out_free_resolver;
-    }
-  else if (event != AVAHI_RESOLVER_FOUND)
-    {
-      DPRINTF(E_LOG, L_MDNS, "Avahi Resolver empty callback\n");
-      goto out_free_resolver;
+      if (event == AVAHI_RESOLVER_FAILURE)
+	DPRINTF(E_LOG, L_MDNS, "Avahi Resolver failure: service '%s' type '%s' proto %d: %s\n", name, type, proto, MDNSERR);
+      else
+	DPRINTF(E_LOG, L_MDNS, "Avahi Resolver empty callback\n");
+
+      family = avahi_proto_to_af(proto);
+      if (family != AF_UNSPEC)
+	mb->cb(name, type, domain, NULL, family, NULL, -1, NULL);
+
+      // We don't clean up resolvers because we want a notification from them if
+      // the service reappears (e.g. if device was switched off and then on)
+
+      return;
     }
 
   DPRINTF(E_DBG, L_MDNS, "Avahi Resolver: resolved service '%s' type '%s' proto %d, host %s\n", name, type, proto, hostname);
 
-  rb_data = calloc(1, sizeof(struct mdns_record_browser));
-  if (! (rb_data && (rb_data->name = strdup(name)) && (rb_data->domain = strdup(domain))) )
-    {
-      DPRINTF(E_LOG, L_MDNS, "Out of memory\n");
-      goto out_free_resolver;
-    }
+  CHECK_NULL(L_MDNS, rb_data = calloc(1, sizeof(struct mdns_record_browser)));
 
-  rb_data->mb = (struct mdns_browser *)userdata;
+  rb_data->name = strdup(name);
+  rb_data->domain = strdup(domain);
+  rb_data->mb = mb;
   rb_data->port = port;
 
   while (txt)
@@ -536,9 +597,6 @@ browse_resolve_callback(AvahiServiceResolver *r, AvahiIfIndex intf, AvahiProtoco
   rb = avahi_record_browser_new(mdns_client, intf, proto, hostname, AVAHI_DNS_CLASS_IN, dns_type, 0, browse_record_callback, rb_data);
   if (!rb)
     DPRINTF(E_LOG, L_MDNS, "Could not create record browser for host %s: %s\n", hostname, MDNSERR);
-
- out_free_resolver:
-  avahi_service_resolver_free(r);
 }
 
 static void
@@ -546,7 +604,7 @@ browse_callback(AvahiServiceBrowser *b, AvahiIfIndex intf, AvahiProtocol proto,
 		const char *name, const char *type, const char *domain, AvahiLookupResultFlags flags, void *userdata)
 {
   struct mdns_browser *mb;
-  AvahiServiceResolver *res;
+  struct mdns_resolver *r;
   int family;
 
   mb = (struct mdns_browser *)userdata;
@@ -560,16 +618,30 @@ browse_callback(AvahiServiceBrowser *b, AvahiIfIndex intf, AvahiProtocol proto,
 
 	b = avahi_service_browser_new(mdns_client, AVAHI_IF_UNSPEC, mb->protocol, mb->type, NULL, 0, browse_callback, mb);
 	if (!b)
-	  DPRINTF(E_LOG, L_MDNS, "Failed to recreate service browser (service type %s): %s\n", mb->type, MDNSERR);
+	  {
+	    DPRINTF(E_LOG, L_MDNS, "Failed to recreate service browser (service type %s): %s\n", mb->type, MDNSERR);
+	    return;
+	  }
 
-	return;
+	break;
 
       case AVAHI_BROWSER_NEW:
 	DPRINTF(E_DBG, L_MDNS, "Avahi Browser: NEW service '%s' type '%s' proto %d\n", name, type, proto);
 
-	res = avahi_service_resolver_new(mdns_client, intf, proto, name, type, domain, proto, 0, browse_resolve_callback, mb);
-	if (!res)
-	  DPRINTF(E_LOG, L_MDNS, "Failed to create service resolver: %s\n", MDNSERR);
+	CHECK_NULL(L_MDNS, r = calloc(1, sizeof(struct mdns_resolver)));
+
+	r->resolver = avahi_service_resolver_new(mdns_client, intf, proto, name, type, domain, proto, 0, browse_resolve_callback, mb);
+	if (!r->resolver)
+	  {
+	    DPRINTF(E_LOG, L_MDNS, "Failed to create service resolver: %s\n", MDNSERR);
+	    free(r);
+	    return;
+	  }
+
+	r->name = strdup(name);
+	r->proto = proto;
+	r->next = resolver_list;
+	resolver_list = r;
 
 	break;
 
@@ -579,6 +651,9 @@ browse_callback(AvahiServiceBrowser *b, AvahiIfIndex intf, AvahiProtocol proto,
 	family = avahi_proto_to_af(proto);
 	if (family != AF_UNSPEC)
 	  mb->cb(name, type, domain, NULL, family, NULL, -1, NULL);
+
+	resolvers_cleanup(name, proto);
+
 	break;
 
       case AVAHI_BROWSER_ALL_FOR_NOW:
@@ -620,13 +695,101 @@ entry_group_callback(AvahiEntryGroup *g, AvahiEntryGroupState state, AVAHI_GCC_U
     }
 }
 
-static void
-_create_services(void)
+static int
+create_group_entry(struct mdns_group_entry *ge, int commit)
 {
-  struct mdns_group_entry *pentry;
+  char hostname[HOST_NAME_MAX + 1];
+  char rdata[HOST_NAME_MAX + 6 + 1]; // Includes room for ".local" and 0-terminator
+  int count;
+  int i;
   int ret;
 
-  DPRINTF(E_DBG, L_MDNS, "Creating service group\n");
+  if (!mdns_group)
+    {
+      mdns_group = avahi_entry_group_new(mdns_client, entry_group_callback, NULL);
+      if (!mdns_group)
+	{
+	  DPRINTF(E_WARN, L_MDNS, "Could not create Avahi EntryGroup: %s\n", MDNSERR);
+	  return -1;
+	}
+    }
+
+  if (ge->publish == MDNS_PUBLISH_SERVICE)
+    {
+      DPRINTF(E_DBG, L_MDNS, "Adding service %s/%s\n", ge->name, ge->type);
+
+      ret = avahi_entry_group_add_service_strlst(mdns_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, 0,
+						 ge->name, ge->type,
+						 NULL, NULL, ge->port, ge->txt);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_MDNS, "Could not add mDNS service %s/%s: %s\n", ge->name, ge->type, avahi_strerror(ret));
+	  return -1;
+	}
+    }
+  else if (ge->publish == MDNS_PUBLISH_CNAME)
+    {
+      DPRINTF(E_DBG, L_MDNS, "Adding CNAME record %s\n", ge->name);
+
+      ret = gethostname(hostname, HOST_NAME_MAX);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_MDNS, "Could not add CNAME %s, gethostname failed\n", ge->name);
+	  return -1;
+	}
+      // Note, gethostname does not guarantee 0-termination
+      hostname[HOST_NAME_MAX] = 0;
+
+      ret = snprintf(rdata, sizeof(rdata), ".%s.local", hostname);
+      if (!(ret > 0 && ret < sizeof(rdata)))
+        {
+	  DPRINTF(E_LOG, L_MDNS, "Could not add CNAME %s, hostname is invalid\n", ge->name);
+	  return -1;
+        }
+
+      // Convert to dns string: .forked-daapd.local -> \12forked-daapd\6local
+      count = 0;
+      for (i = ret - 1; i >= 0; i--)
+        {
+	  if (rdata[i] == '.')
+	    {
+	      rdata[i] = count;
+	      count = 0;
+	    }
+	  else
+	    count++;
+        }
+
+      // ret + 1 should be the string length of rdata incl. 0-terminator
+      ret = avahi_entry_group_add_record(mdns_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC,
+                                         AVAHI_PUBLISH_USE_MULTICAST | AVAHI_PUBLISH_ALLOW_MULTIPLE,
+                                         ge->name, AVAHI_DNS_CLASS_IN, AVAHI_DNS_TYPE_CNAME,
+                                         AVAHI_DEFAULT_TTL, rdata, ret + 1);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_MDNS, "Could not add CNAME record %s: %s\n", ge->name, avahi_strerror(ret));
+	  return -1;
+	}
+    }
+
+  if (!commit)
+    return 0;
+
+  ret = avahi_entry_group_commit(mdns_group);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_MDNS, "Could not commit mDNS services: %s\n", MDNSERR);
+      return -1;
+    }
+
+  return 0;
+}
+
+static void
+create_all_group_entries(void)
+{
+  struct mdns_group_entry *ge;
+  int ret;
 
   if (!group_entries)
     {
@@ -634,36 +797,21 @@ _create_services(void)
       return;
     }
 
-    if (mdns_group == NULL)
-      {
-        mdns_group = avahi_entry_group_new(mdns_client, entry_group_callback, NULL);
-	if (!mdns_group)
-	  {
-            DPRINTF(E_WARN, L_MDNS, "Could not create Avahi EntryGroup: %s\n", MDNSERR);
-            return;
-	  }
-      }
+  if (mdns_group)
+    avahi_entry_group_reset(mdns_group);
 
-    pentry = group_entries;
-    while (pentry)
-      {
-        DPRINTF(E_DBG, L_MDNS, "Re-registering %s/%s\n", pentry->name, pentry->type);
+  DPRINTF(E_INFO, L_MDNS, "Re-registering mDNS groups (services and records)\n");
 
-        ret = avahi_entry_group_add_service_strlst(mdns_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, 0,
-						   pentry->name, pentry->type,
-						   NULL, NULL, pentry->port, pentry->txt);
-	if (ret < 0)
-	  {
-	    DPRINTF(E_WARN, L_MDNS, "Could not add mDNS services: %s\n", avahi_strerror(ret));
-	    return;
-	  }
+  for (ge = group_entries; ge; ge = ge->next)
+    {
+      create_group_entry(ge, 0);
+      if (!mdns_group)
+	return;
+    }
 
-	pentry = pentry->next;
-      }
-
-    ret = avahi_entry_group_commit(mdns_group);
-    if (ret < 0)
-      DPRINTF(E_WARN, L_MDNS, "Could not commit mDNS services: %s\n", MDNSERR);
+  ret = avahi_entry_group_commit(mdns_group);
+  if (ret < 0)
+    DPRINTF(E_WARN, L_MDNS, "Could not commit mDNS services: %s\n", MDNSERR);
 }
 
 static void
@@ -678,7 +826,7 @@ client_callback(AvahiClient *c, AvahiClientState state, AVAHI_GCC_UNUSED void *
       case AVAHI_CLIENT_S_RUNNING:
         DPRINTF(E_LOG, L_MDNS, "Avahi state change: Client running\n");
         if (!mdns_group)
-	  _create_services();
+	  create_all_group_entries();
 
 	for (mb = browser_list; mb; mb = mb->next)
 	  {
@@ -806,8 +954,6 @@ mdns_register(char *name, char *type, int port, char **txt)
   AvahiStringList *txt_sl;
   int i;
 
-  DPRINTF(E_DBG, L_MDNS, "Adding mDNS service %s/%s\n", name, type);
-
   ge = calloc(1, sizeof(struct mdns_group_entry));
   if (!ge)
     {
@@ -815,6 +961,7 @@ mdns_register(char *name, char *type, int port, char **txt)
       return -1;
     }
 
+  ge->publish = MDNS_PUBLISH_SERVICE;
   ge->name = strdup(name);
   ge->type = strdup(type);
   ge->port = port;
@@ -835,14 +982,30 @@ mdns_register(char *name, char *type, int port, char **txt)
   ge->next = group_entries;
   group_entries = ge;
 
-  if (mdns_group)
+  create_all_group_entries(); // TODO why is this required?
+
+  return 0;
+}
+
+int
+mdns_cname(char *name)
+{
+  struct mdns_group_entry *ge;
+
+  ge = calloc(1, sizeof(struct mdns_group_entry));
+  if (!ge)
     {
-      DPRINTF(E_DBG, L_MDNS, "Resetting mDNS group\n");
-      avahi_entry_group_reset(mdns_group);
+      DPRINTF(E_LOG, L_MDNS, "Out of memory for mDNS CNAME\n");
+      return -1;
     }
 
-  DPRINTF(E_DBG, L_MDNS, "Creating service group\n");
-  _create_services();
+  ge->publish = MDNS_PUBLISH_CNAME;
+  ge->name = strdup(name);
+
+  ge->next = group_entries;
+  group_entries = ge;
+
+  create_all_group_entries();
 
   return 0;
 }
