@@ -27,12 +27,17 @@
 #endif
 
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <limits.h>
 #include <sys/param.h>
+#ifndef CLOCK_REALTIME
+#include <sys/time.h>
+#endif
 
 #include <unistr.h>
 #include <uniconv.h>
@@ -47,6 +52,8 @@ safe_atoi32(const char *str, int32_t *val)
   char *end;
   long intval;
 
+  *val = 0;
+
   errno = 0;
   intval = strtol(str, &end, 10);
 
@@ -83,6 +90,8 @@ safe_atou32(const char *str, uint32_t *val)
   char *end;
   unsigned long intval;
 
+  *val = 0;
+
   errno = 0;
   intval = strtoul(str, &end, 10);
 
@@ -119,6 +128,8 @@ safe_hextou32(const char *str, uint32_t *val)
   char *end;
   unsigned long intval;
 
+  *val = 0;
+
   /* A hex shall begin with 0x */
   if (strncmp(str, "0x", 2) != 0)
     return safe_atou32(str, val);
@@ -159,6 +170,8 @@ safe_atoi64(const char *str, int64_t *val)
   char *end;
   long long intval;
 
+  *val = 0;
+
   errno = 0;
   intval = strtoll(str, &end, 10);
 
@@ -195,6 +208,8 @@ safe_atou64(const char *str, uint64_t *val)
   char *end;
   unsigned long long intval;
 
+  *val = 0;
+
   errno = 0;
   intval = strtoull(str, &end, 10);
 
@@ -231,6 +246,8 @@ safe_hextou64(const char *str, uint64_t *val)
   char *end;
   unsigned long long intval;
 
+  *val = 0;
+
   errno = 0;
   intval = strtoull(str, &end, 16);
 
@@ -261,6 +278,36 @@ safe_hextou64(const char *str, uint64_t *val)
   return 0;
 }
 
+char *
+safe_strdup(const char *str)
+{
+  if (str == NULL)
+    return NULL;
+
+  return strdup(str);
+}
+
+/*
+ * Wrapper function for vasprintf by Intel Corporation
+ * Published under the L-GPL 2.1 licence as part of clr-boot-manager
+ *
+ * https://github.com/clearlinux/clr-boot-manager
+ */
+char *
+safe_asprintf(const char *fmt, ...)
+{
+  char *ret = NULL;
+  va_list va;
+  va_start(va, fmt);
+  if (vasprintf(&ret, fmt, va) < 0)
+    {
+      DPRINTF(E_FATAL, L_MISC, "Out of memory for safe_asprintf\n");
+      abort();
+    }
+  va_end(va);
+  return ret;
+}
+
 
 /* Key/value functions */
 struct keyval *
@@ -430,7 +477,7 @@ keyval_sort(struct keyval *kv)
   struct onekeyval *okv;
   struct onekeyval *sokv;
 
-  if (!kv)
+  if (!kv || !kv->head)
     return;
 
   head = kv->head;
@@ -486,6 +533,57 @@ m_realpath(const char *pathname)
   return ret;
 }
 
+char **
+m_readfile(const char *path, int num_lines)
+{
+  char buf[256];
+  FILE *fp;
+  char **lines;
+  char *line;
+  int i;
+
+  // Alloc array of char pointers
+  lines = calloc(num_lines, sizeof(char *));
+  if (!lines)
+    return NULL;
+
+  fp = fopen(path, "rb");
+  if (!fp)
+    {
+      DPRINTF(E_LOG, L_MISC, "Could not open file '%s' for reading: %s\n", path, strerror(errno));
+      free(lines);
+      return NULL;
+    }
+
+  for (i = 0; i < num_lines; i++)
+    {
+      line = fgets(buf, sizeof(buf), fp);
+      if (!line)
+	{
+	  DPRINTF(E_LOG, L_MISC, "File '%s' has fewer lines than expected (found %d, expected %d)\n", path, i, num_lines);
+	  goto error;
+	}
+
+      lines[i] = trimwhitespace(line);
+      if (!lines[i] || (strlen(lines[i]) == 0))
+	{
+	  DPRINTF(E_LOG, L_MISC, "Line %d in '%s' is invalid\n", i+1, path);
+	  goto error;
+	}
+    }
+
+  fclose(fp);
+
+  return lines;
+
+ error:
+  for (i = 0; i < num_lines; i++)
+    free(lines[i]);
+
+  free(lines);
+  fclose(fp);
+  return NULL;
+}
 
 char *
 unicode_fixup_string(char *str, const char *fromcode)
@@ -559,10 +657,18 @@ trimwhitespace(const char *str)
   return out;
 }
 
-uint32_t
-djb_hash(void *data, size_t len)
+void
+swap_pointers(char **a, char **b)
 {
-  unsigned char *bytes = data;
+  char *t = *a;
+  *a = *b;
+  *b = t;
+}
+
+uint32_t
+djb_hash(const void *data, size_t len)
+{
+  const unsigned char *bytes = data;
   uint32_t hash = 5381;
 
   while (len--)
@@ -668,7 +774,7 @@ b64_decode(const char *b64)
 static const char b64_encode_table[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 static void
-b64_encode_block(uint8_t *in, char *out, int len)
+b64_encode_block(const uint8_t *in, char *out, int len)
 {
   out[0] = b64_encode_table[in[0] >> 2];
 
@@ -691,7 +797,7 @@ b64_encode_block(uint8_t *in, char *out, int len)
 }
 
 static void
-b64_encode_full_block(uint8_t *in, char *out)
+b64_encode_full_block(const uint8_t *in, char *out)
 {
   out[0] = b64_encode_table[in[0] >> 2];
   out[1] = b64_encode_table[((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4)];
@@ -700,7 +806,7 @@ b64_encode_full_block(uint8_t *in, char *out)
 }
 
 char *
-b64_encode(uint8_t *in, size_t len)
+b64_encode(const uint8_t *in, size_t len)
 {
   char *encoded;
   char *out;
@@ -775,17 +881,17 @@ murmur_hash64(const void *key, int len, uint32_t seed)
   switch (len & 7)
     {
       case 7:
-	h ^= (uint64_t)(data_tail[6]) << 48;
+	h ^= (uint64_t)(data_tail[6]) << 48; /* FALLTHROUGH */
       case 6:
-	h ^= (uint64_t)(data_tail[5]) << 40;
+	h ^= (uint64_t)(data_tail[5]) << 40; /* FALLTHROUGH */
       case 5:
-	h ^= (uint64_t)(data_tail[4]) << 32;
+	h ^= (uint64_t)(data_tail[4]) << 32; /* FALLTHROUGH */
       case 4:
-	h ^= (uint64_t)(data_tail[3]) << 24;
+	h ^= (uint64_t)(data_tail[3]) << 24; /* FALLTHROUGH */
       case 3:
-	h ^= (uint64_t)(data_tail[2]) << 16;
+	h ^= (uint64_t)(data_tail[2]) << 16; /* FALLTHROUGH */
       case 2:
-	h ^= (uint64_t)(data_tail[1]) << 8;
+	h ^= (uint64_t)(data_tail[1]) << 8; /* FALLTHROUGH */
       case 1:
 	h ^= (uint64_t)(data_tail[0]);
 	h *= m;
@@ -918,3 +1024,186 @@ timespec_cmp(struct timespec time1, struct timespec time2)
   else
     return 0;
 }
+
+struct timespec
+timespec_reltoabs(struct timespec relative)
+{
+  struct timespec absolute;
+
+#ifdef CLOCK_REALTIME
+  clock_gettime(CLOCK_REALTIME, &absolute);
+#else
+  struct timeval tv;
+  gettimeofday(&tv, NULL);
+  TIMEVAL_TO_TIMESPEC(&tv, &absolute);
+#endif
+  return timespec_add(absolute, relative);
+}
+
+#if defined(HAVE_MACH_CLOCK) || defined(HAVE_MACH_TIMER)
+
+#include <mach/mach_time.h> /* mach_absolute_time */
+#include <mach/mach.h>      /* host_get_clock_service */
+#include <mach/clock.h>     /* clock_get_time */
+
+/* mach monotonic clock port */
+extern mach_port_t clock_port;
+
+#ifndef HAVE_CLOCK_GETTIME
+
+int
+clock_gettime(clockid_t clock_id, struct timespec *tp)
+{
+  static int clock_init = 0;
+  static clock_serv_t clock;
+
+  mach_timespec_t mts;
+  int ret;
+
+  if (! clock_init) {
+    clock_init = 1;
+    if (host_get_clock_service(mach_host_self(), CALENDAR_CLOCK, &clock))
+      abort(); /* unlikely */
+  }
+
+  if(! tp)
+    return -1;
+
+  switch (clock_id) {
+
+  case CLOCK_REALTIME:
+
+    /* query mach for calendar time */
+    ret = clock_get_time(clock, &mts);
+    if (! ret) {
+      tp->tv_sec = mts.tv_sec;
+      tp->tv_nsec = mts.tv_nsec;
+    }
+    break;
+
+  case CLOCK_MONOTONIC:
+
+    /* query mach for monotinic time */
+    ret = clock_get_time(clock_port, &mts);
+    if (! ret) {
+      tp->tv_sec = mts.tv_sec;
+      tp->tv_nsec = mts.tv_nsec;
+    }
+    break;
+
+  default:
+    ret = -1;
+    break;
+  }
+
+  return ret;
+}
+
+int
+clock_getres(clockid_t clock_id, struct timespec *res)
+{
+  if (! res)
+    return -1;
+
+  /* hardcode ms resolution */
+  res->tv_sec = 0;
+  res->tv_nsec = 1000;
+
+  return 0;
+}
+
+#endif /* HAVE_CLOCK_GETTIME */
+
+#ifndef HAVE_TIMER_SETTIME
+
+#include <sys/time.h> /* ITIMER_REAL */
+
+int
+timer_create(clockid_t clock_id, void *sevp, timer_t *timer_id)
+{
+  if (clock_id != CLOCK_MONOTONIC)
+    return -1;
+  if (sevp)
+    return -1;
+
+  /* setitimer only supports one timer */
+  *timer_id = 0;
+
+  return 0;
+}
+
+int
+timer_delete(timer_t timer_id)
+{
+  struct itimerval timerval;
+
+  if (timer_id != 0)
+    return -1;
+
+  memset(&timerval, 0, sizeof(struct itimerval));
+
+  return setitimer(ITIMER_REAL, &timerval, NULL);
+}
+
+int
+timer_settime(timer_t timer_id, int flags, const struct itimerspec *tp, struct itimerspec *old)
+{
+  struct itimerval tv;
+
+  if (timer_id != 0 || ! tp || old)
+    return -1;
+
+  TIMESPEC_TO_TIMEVAL(&(tv.it_value), &(tp->it_value));
+  TIMESPEC_TO_TIMEVAL(&(tv.it_interval), &(tp->it_interval));
+
+  return setitimer(ITIMER_REAL, &tv, NULL);
+}
+
+int
+timer_getoverrun(timer_t timer_id)
+{
+  /* since we don't know if there have been signals that weren't delivered,
+     assume none */
+  return 0;
+}
+
+#endif /* HAVE_TIMER_SETTIME */
+
+#endif /* HAVE_MACH_CLOCK */
+
+int
+mutex_init(pthread_mutex_t *mutex)
+{
+  pthread_mutexattr_t mattr;
+  int err;
+
+  CHECK_ERR(L_MISC, pthread_mutexattr_init(&mattr));
+  CHECK_ERR(L_MISC, pthread_mutexattr_settype(&mattr, PTHREAD_MUTEX_ERRORCHECK));
+  err = pthread_mutex_init(mutex, &mattr);
+  CHECK_ERR(L_MISC, pthread_mutexattr_destroy(&mattr));
+
+  return err;
+}
+
+void
+log_fatal_err(int domain, const char *func, int line, int err)
+{
+  DPRINTF(E_FATAL, domain, "%s failed at line %d, error %d (%s)\n", func, line, err, strerror(err));
+  abort();
+}
+
+void
+log_fatal_errno(int domain, const char *func, int line)
+{
+  DPRINTF(E_FATAL, domain, "%s failed at line %d, error %d (%s)\n", func, line, errno, strerror(errno));
+  abort();
+}
+
+void
+log_fatal_null(int domain, const char *func, int line)
+{
+  DPRINTF(E_FATAL, domain, "%s returned NULL at line %d\n", func, line);
+  abort();
+}
+
+
