@@ -32,6 +32,7 @@
 #include <event2/event.h>
 #include <asoundlib.h>
 
+#include "misc.h"
 #include "conffile.h"
 #include "logger.h"
 #include "player.h"
@@ -170,10 +171,17 @@ alsa_session_make(struct output_device *device, output_status_cb cb)
   struct alsa_session *as;
 
   os = calloc(1, sizeof(struct output_session));
-  as = calloc(1, sizeof(struct alsa_session));
-  if (!os || !as)
+  if (!os)
     {
-      DPRINTF(E_LOG, L_LAUDIO, "Out of memory for ALSA session\n");
+      DPRINTF(E_LOG, L_LAUDIO, "Out of memory for ALSA session (os)\n");
+      return NULL;
+    }
+
+  as = calloc(1, sizeof(struct alsa_session));
+  if (!as)
+    {
+      DPRINTF(E_LOG, L_LAUDIO, "Out of memory for ALSA session (as)\n");
+      free(os);
       return NULL;
     }
 
@@ -803,7 +811,10 @@ alsa_device_start(struct output_device *device, output_status_cb cb, uint64_t rt
 
   ret = device_open(as);
   if (ret < 0)
-    return -1;
+    {
+      alsa_session_cleanup(as);
+      return -1;
+    }
 
   as->state = ALSA_STATE_STARTED;
   alsa_status(as);
@@ -1009,7 +1020,7 @@ alsa_init(void)
     }
 
   device->id = 0;
-  device->name = nickname;
+  device->name = strdup(nickname);
   device->type = OUTPUT_TYPE_ALSA;
   device->type_name = outputs_name(device->type);
   device->advertised = 1;
