@@ -30,18 +30,23 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
+#include <net/if.h>
+#include <netinet/in.h>
 #include <ifaddrs.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <endian.h>
+#ifdef HAVE_ENDIAN_H
+# include <endian.h>
+#elif defined(HAVE_SYS_ENDIAN_H)
+# include <sys/endian.h>
+#elif defined(HAVE_LIBKERN_OSBYTEORDER_H)
+#include <libkern/OSByteOrder.h>
+#define htobe32(x) OSSwapHostToBigInt32(x)
+#define be32toh(x) OSSwapBigToHostInt32(x)
+#endif
 #include <gnutls/gnutls.h>
 #include <event2/event.h>
-
-#ifdef HAVE_JSON_C_OLD
-# include <json/json.h>
-#else
-# include <json-c/json.h>
-#endif
+#include <json.h>
 
 #include "conffile.h"
 #include "mdns.h"
@@ -368,7 +373,11 @@ tcp_connect(const char *address, unsigned int port, int family)
 
   // TODO Open non-block right away so we don't block the player while connecting
   // and during TLS handshake (we would probably need to introduce a deferredev)
+#ifdef SOCK_CLOEXEC
   fd = socket(family, SOCK_STREAM | SOCK_CLOEXEC, 0);
+#else
+  fd = socket(family, SOCK_STREAM, 0);
+#endif
   if (fd < 0)
     {
       DPRINTF(E_LOG, L_CAST, "Could not create socket: %s\n", strerror(errno));
@@ -1220,6 +1229,7 @@ static void
 cast_device_cb(const char *name, const char *type, const char *domain, const char *hostname, int family, const char *address, int port, struct keyval *txt)
 {
   struct output_device *device;
+  const char *friendly_name;
   uint32_t id;
 
   id = djb_hash(name, strlen(name));
@@ -1229,6 +1239,10 @@ cast_device_cb(const char *name, const char *type, const char *domain, const cha
       return;
     }
 
+  friendly_name = keyval_get(txt, "fn");
+  if (friendly_name)
+    name = friendly_name;
+
   DPRINTF(E_DBG, L_CAST, "Event for Chromecast device '%s' (port %d, id %" PRIu32 ")\n", name, port, id);
 
   device = calloc(1, sizeof(struct output_device));
@@ -1323,10 +1337,17 @@ cast_session_make(struct output_device *device, int family, output_status_cb cb)
     }
 
   os = calloc(1, sizeof(struct output_session));
-  cs = calloc(1, sizeof(struct cast_session));
-  if (!os || !cs)
+  if (!os)
     {
-      DPRINTF(E_LOG, L_CAST, "Out of memory for TLS session\n");
+      DPRINTF(E_LOG, L_CAST, "Out of memory (os)\n");
+      return NULL;
+    }
+
+  cs = calloc(1, sizeof(struct cast_session));
+  if (!cs)
+    {
+      DPRINTF(E_LOG, L_CAST, "Out of memory (cs)\n");
+      free(os);
       return NULL;
     }
 
@@ -1450,6 +1471,8 @@ cast_session_shutdown(struct cast_session *cs, enum cast_state wanted_state)
 	if ((ret < 0) || (wanted_state >= CAST_STATE_MEDIA_LAUNCHED))
 	  break;
 
+	/* FALLTHROUGH */
+
       case CAST_STATE_MEDIA_LAUNCHED:
 	ret = cast_msg_send(cs, STOP, cast_cb_stop);
 	pending = 1;
