@@ -5,7 +5,6 @@
 #include <stdint.h>
 
 #include "db.h"
-#include "queue.h"
 
 /* AirTunes v2 packet interval in ns */
 /* (352 samples/packet * 1e9 ns/s) / 44100 samples/s = 7981859 ns/packet */
@@ -14,11 +13,6 @@
 /* AirTunes v2 number of samples per packet */
 #define AIRTUNES_V2_PACKET_SAMPLES  352
 
-
-/* Samples to bytes, bytes to samples */
-#define STOB(s) ((s) * 4)
-#define BTOS(b) ((b) / 4)
-
 /* Maximum number of previously played songs that are remembered */
 #define MAX_HISTORY_COUNT 20
 
@@ -28,6 +22,12 @@ enum play_status {
   PLAY_PLAYING = 4,
 };
 
+enum repeat_mode {
+  REPEAT_OFF  = 0,
+  REPEAT_SONG = 1,
+  REPEAT_ALL  = 2,
+};
+
 struct spk_flags {
   unsigned selected:1;
   unsigned has_password:1;
@@ -39,18 +39,12 @@ struct player_status {
   enum play_status status;
   enum repeat_mode repeat;
   char shuffle;
+  char consume;
 
   int volume;
 
   /* Playlist id */
   uint32_t plid;
-  /* Playlist version
-     After startup plversion is 0 and gets incremented after each change of the playlist
-     (e. g. after adding/moving/removing items). It is used by mpd clients to recognize if
-     they need to update the current playlist. */
-  uint32_t plversion;
-  /* Playlist length */
-  uint32_t playlistlength;
   /* Id of the playing file/item in the files database */
   uint32_t id;
   /* Item-Id of the playing file/item in the queue */
@@ -59,14 +53,6 @@ struct player_status {
   uint32_t pos_ms;
   /* Length in ms of playing item */
   uint32_t len_ms;
-  /* Playlist position of playing item*/
-  int pos_pl;
-  /* Item id of next item in playlist */
-  uint32_t next_id;
-  /* Item-Id of the next file/item in the queue */
-  uint32_t next_item_id;
-  /* Playlist position of next item */
-  int next_pos_pl;
 };
 
 typedef void (*spk_enum_cb)(uint64_t id, const char *name, int relvol, int absvol, struct spk_flags flags, void *arg);
@@ -94,9 +80,6 @@ player_get_status(struct player_status *status);
 int
 player_now_playing(uint32_t *id);
 
-char *
-player_get_icy_artwork_url(uint32_t id);
-
 void
 player_speaker_enumerate(spk_enum_cb cb, void *arg);
 
@@ -104,16 +87,13 @@ int
 player_speaker_set(uint64_t *ids);
 
 int
-player_playback_start(uint32_t *id);
+player_playback_start(void);
 
 int
-player_playback_start_byindex(int pos, uint32_t *id);
+player_playback_start_byitem(struct db_queue_item *queue_item);
 
 int
-player_playback_start_bypos(int pos, uint32_t *id);
-
-int
-player_playback_start_byitemid(uint32_t item_id, uint32_t *id);
+player_playback_start_byid(uint32_t id);
 
 int
 player_playback_stop(void);
@@ -130,7 +110,6 @@ player_playback_next(void);
 int
 player_playback_prev(void);
 
-
 int
 player_volume_set(int vol);
 
@@ -146,39 +125,9 @@ player_repeat_set(enum repeat_mode mode);
 int
 player_shuffle_set(int enable);
 
-
-struct queue *
-player_queue_get_bypos(int count);
-
-struct queue *
-player_queue_get_byindex(int pos, int count);
-
 int
-player_queue_add(struct queue_item *items, uint32_t *item_id);
+player_consume_set(int enable);
 
-int
-player_queue_add_next(struct queue_item *items);
-
-int
-player_queue_move_bypos(int ps_pos_from, int ps_pos_to);
-
-int
-player_queue_move_byindex(int pos_from, int pos_to);
-
-int
-player_queue_move_byitemid(uint32_t item_id, int pos_to);
-
-int
-player_queue_remove_bypos(int pos);
-
-int
-player_queue_remove_byindex(int pos, int count);
-
-int
-player_queue_remove_byitemid(uint32_t id);
-
-void
-player_queue_clear(void);
 
 void
 player_queue_clear_history(void);
@@ -186,14 +135,20 @@ player_queue_clear_history(void);
 void
 player_queue_plid(uint32_t plid);
 
+struct player_history *
+player_history_get(void);
+
 int
 player_device_add(void *device);
 
 int
 player_device_remove(void *device);
 
-struct player_history *
-player_history_get(void);
+void
+player_raop_verification_kickoff(char **arglist);
+
+void
+player_metadata_send(void *imd, void *omd);
 
 int
 player_init(void);
