@@ -39,10 +39,8 @@
 #include <stdint.h>
 #include <inttypes.h>
 #include <errno.h>
-#include <pthread.h>
 
-#if defined(HAVE_SYS_EVENTFD_H) && defined(HAVE_EVENTFD)
-# define USE_EVENTFD
+#ifdef HAVE_EVENTFD
 # include <sys/eventfd.h>
 #endif
 
@@ -80,14 +78,14 @@ struct remote_info {
 /* Main event base, from main.c */
 extern struct event_base *evbase_main;
 
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
 static int pairing_efd;
 #else
 static int pairing_pipe[2];
 #endif
 static struct event *pairingev;
-static pthread_mutex_t remote_lck = PTHREAD_MUTEX_INITIALIZER;
-static struct remote_info *remote_list;
+static pthread_mutex_t remote_lck;
+static struct remote_info *remote_info;
 
 
 /* iTunes - Remote pairing hash */
@@ -156,45 +154,27 @@ itunes_pairing_hash(char *paircode, char *pin)
  * with the list lock held by the caller
  */
 static struct remote_info *
-add_remote(void)
+create_remote(void)
 {
   struct remote_info *ri;
 
-  ri = (struct remote_info *)malloc(sizeof(struct remote_info));
+  ri = calloc(1, sizeof(struct remote_info));
   if (!ri)
     {
       DPRINTF(E_WARN, L_REMOTE, "Out of memory for struct remote_info\n");
       return NULL;
     }
 
-  memset(ri, 0, sizeof(struct remote_info));
-
-  ri->next = remote_list;
-  remote_list = ri;
-
   return ri;
 }
 
 static void
 unlink_remote(struct remote_info *ri)
 {
-  struct remote_info *p;
-
-  if (ri == remote_list)
-    remote_list = ri->next;
+  if (ri == remote_info)
+    remote_info = NULL;
   else
-    {
-      for (p = remote_list; p && (p->next != ri); p = p->next)
-	; /* EMPTY */
-
-      if (!p)
-	{
-	  DPRINTF(E_LOG, L_REMOTE, "WARNING: struct remote_info not found in list; BUG!\n");
-	  return;
-	}
-
-      p->next = ri->next;
-    }
+    DPRINTF(E_LOG, L_REMOTE, "WARNING: struct remote_info not found in list; BUG!\n");
 }
 
 static void
@@ -228,16 +208,10 @@ remove_remote(struct remote_info *ri)
 static void
 remove_remote_address_byid(const char *id, int family)
 {
-  struct remote_info *ri;
+  struct remote_info *ri = NULL;
 
-  for (ri = remote_list; ri; ri = ri->next)
-    {
-      if (!ri->pi.remote_id)
-	continue;
-
-      if (strcmp(ri->pi.remote_id, id) == 0)
-	break;
-    }
+  if (remote_info && strcmp(remote_info->pi.remote_id, id) == 0)
+    ri = remote_info;
 
   if (!ri)
     {
@@ -271,70 +245,48 @@ remove_remote_address_byid(const char *id, int family)
 static int
 add_remote_mdns_data(const char *id, int family, const char *address, int port, char *name, char *paircode)
 {
-  struct remote_info *ri;
   char *check_addr;
   int ret;
 
-  for (ri = remote_list; ri; ri = ri->next)
+  if (remote_info && strcmp(remote_info->pi.remote_id, id) == 0)
     {
-      if (!ri->pi.remote_id)
-	continue;
-
-      if (strcmp(ri->pi.remote_id, id) == 0)
-	break;
-    }
-
-  if (!ri)
-    {
-      DPRINTF(E_DBG, L_REMOTE, "Remote id %s not known, adding\n", id);
-
-      ri = add_remote();
-      if (!ri)
-	return -1;
-
-      ret = 0;
+      DPRINTF(E_DBG, L_REMOTE, "Remote id %s found\n", id);
+      free_pi(&remote_info->pi, 1);
+      ret = 1;
     }
   else
     {
-      DPRINTF(E_DBG, L_REMOTE, "Remote id %s found\n", id);
-
-      free_pi(&ri->pi, 1);
-
-      switch (family)
+      DPRINTF(E_DBG, L_REMOTE, "Remote id %s not known, adding\n", id);
+      if (remote_info)
 	{
-	  case AF_INET:
-	    if (ri->v4_address)
-	      free(ri->v4_address);
-	    break;
-
-	  case AF_INET6:
-	    if (ri->v6_address)
-	      free(ri->v6_address);
-	    break;
+	  DPRINTF(E_DBG, L_REMOTE, "Removing existing remote with id %s\n", remote_info->pi.remote_id);
+	  remove_remote(remote_info);
 	}
 
-      if (ri->paircode)
-	free(ri->paircode);
-
-      ret = 1;
+      remote_info = create_remote();
+      ret = 0;
     }
 
-  ri->pi.remote_id = strdup(id);
+  free(remote_info->paircode);
+  free(remote_info->pi.remote_id);
+  remote_info->pi.remote_id = strdup(id);
 
   switch (family)
     {
       case AF_INET:
-	ri->v4_address = strdup(address);
-	ri->v4_port = port;
+	free(remote_info->v4_address);
+	remote_info->v4_address = strdup(address);
+	remote_info->v4_port = port;
 
-	check_addr = ri->v4_address;
+	check_addr = remote_info->v4_address;
 	break;
 
       case AF_INET6:
-	ri->v6_address = strdup(address);
-	ri->v6_port = port;
+	free(remote_info->v6_address);
+	remote_info->v6_address = strdup(address);
+	remote_info->v6_port = port;
 
-	check_addr = ri->v6_address;
+	check_addr = remote_info->v6_address;
 	break;
 
       default:
@@ -344,44 +296,34 @@ add_remote_mdns_data(const char *id, int family, const char *address, int port,
 	break;
     }
 
-  if (!ri->pi.remote_id || !check_addr)
+  if (!remote_info->pi.remote_id || !check_addr)
     {
       DPRINTF(E_LOG, L_REMOTE, "Out of memory for remote pairing data\n");
 
-      remove_remote(ri);
+      remove_remote(remote_info);
       return -1;
     }
 
-  ri->pi.name = name;
-  ri->paircode = paircode;
+  remote_info->pi.name = name;
+  remote_info->paircode = paircode;
 
   return ret;
 }
 
 static int
-add_remote_pin_data(char *devname, char *pin)
+add_remote_pin_data(const char *pin)
 {
-  struct remote_info *ri;
-
-  for (ri = remote_list; ri; ri = ri->next)
+  if (!remote_info)
     {
-      if (strcmp(ri->pi.name, devname) == 0)
-	break;
-    }
-
-  if (!ri)
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Remote '%s' not known from mDNS, ignoring\n", devname);
+      DPRINTF(E_LOG, L_REMOTE, "No remote known from mDNS, ignoring\n");
 
       return -1;
     }
 
-  DPRINTF(E_DBG, L_REMOTE, "Remote '%s' found\n", devname);
+  DPRINTF(E_DBG, L_REMOTE, "Adding pin to remote '%s'\n", remote_info->pi.name);
 
-  if (ri->pin)
-    free(ri->pin);
-
-  ri->pin = pin;
+  free(remote_info->pin);
+  remote_info->pin = strdup(pin);
 
   return 0;
 }
@@ -389,7 +331,7 @@ add_remote_pin_data(char *devname, char *pin)
 static void
 kickoff_pairing(void)
 {
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   int ret;
 
   ret = eventfd_write(pairing_efd, 1);
@@ -642,7 +584,7 @@ pairing_cb(int fd, short event, void *arg)
 {
   struct remote_info *ri;
 
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   eventfd_t count;
   int ret;
 
@@ -660,28 +602,17 @@ pairing_cb(int fd, short event, void *arg)
     ; /* EMPTY */
 #endif
 
-  for (;;)
+  CHECK_ERR(L_REMOTE, pthread_mutex_lock(&remote_lck));
+
+  if (remote_info && remote_info->paircode && remote_info->pin)
     {
-      pthread_mutex_lock(&remote_lck);
-
-      for (ri = remote_list; ri; ri = ri->next)
-	{
-	  /* We've got both the mDNS data and the pin */
-	  if (ri->paircode && ri->pin)
-	    {
-	      unlink_remote(ri);
-	      break;
-	    }
-	}
-
-      pthread_mutex_unlock(&remote_lck);
-
-      if (!ri)
-	break;
-
+      ri = remote_info;
+      unlink_remote(ri);
       do_pairing(ri);
     }
 
+  CHECK_ERR(L_REMOTE, pthread_mutex_unlock(&remote_lck));
+
   event_add(pairingev, NULL);
 }
 
@@ -701,11 +632,11 @@ touch_remote_cb(const char *name, const char *type, const char *domain, const ch
        * failed; any subsequent attempt will need a new pairing pin, so
        * we can just forget everything we know about the remote.
        */
-      pthread_mutex_lock(&remote_lck);
+      CHECK_ERR(L_REMOTE, pthread_mutex_lock(&remote_lck));
 
       remove_remote_address_byid(name, family);
 
-      pthread_mutex_unlock(&remote_lck);
+      CHECK_ERR(L_REMOTE, pthread_mutex_unlock(&remote_lck));
     }
   else
     {
@@ -763,7 +694,7 @@ touch_remote_cb(const char *name, const char *type, const char *domain, const ch
       DPRINTF(E_LOG, L_REMOTE, "Discovered remote '%s' (id %s) at %s:%d, paircode %s\n", devname, name, address, port, paircode);
 
       /* Add the data to the list, adding the remote to the list if needed */
-      pthread_mutex_lock(&remote_lck);
+      CHECK_ERR(L_REMOTE, pthread_mutex_lock(&remote_lck));
 
       ret = add_remote_mdns_data(name, family, address, port, devname, paircode);
 
@@ -777,126 +708,32 @@ touch_remote_cb(const char *name, const char *type, const char *domain, const ch
       else if (ret == 1)
 	kickoff_pairing();
 
-      pthread_mutex_unlock(&remote_lck);
+      CHECK_ERR(L_REMOTE, pthread_mutex_unlock(&remote_lck));
     }
 }
 
-/* Thread: filescanner */
+/* Thread: filescanner, mpd */
 void
-remote_pairing_read_pin(char *path)
+remote_pairing_kickoff(char **arglist)
 {
-  char buf[256];
-  FILE *fp;
-  char *devname;
-  char *pin;
-  int len;
   int ret;
 
-  fp = fopen(path, "rb");
-  if (!fp)
+  ret = strlen(arglist[0]);
+  if (ret != 4)
     {
-      DPRINTF(E_LOG, L_REMOTE, "Could not open Remote pairing file %s: %s\n", path, strerror(errno));
+      DPRINTF(E_LOG, L_REMOTE, "Kickoff pairing failed, first line did not contain a 4-digit pin (got %d)\n", ret);
       return;
     }
 
-  devname = fgets(buf, sizeof(buf), fp);
-  if (!devname)
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Empty Remote pairing file %s\n", path);
+  DPRINTF(E_LOG, L_REMOTE, "Kickoff pairing with pin '%s'\n", arglist[0]);
 
-      fclose(fp);
-      return;
-    }
+  CHECK_ERR(L_REMOTE, pthread_mutex_lock(&remote_lck));
 
-  len = strlen(devname);
-  if (buf[len - 1] != '\n')
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Invalid Remote pairing file %s: device name too long or missing pin\n", path);
-
-      fclose(fp);
-      return;
-    }
-
-  while (len)
-    {
-      if ((buf[len - 1] == '\r') || (buf[len - 1] == '\n'))
-	{
-	  buf[len - 1] = '\0';
-	  len--;
-	}
-      else
-	break;
-    }
-
-  if (!len)
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Invalid Remote pairing file %s: empty line where device name expected\n", path);
-
-      fclose(fp);
-      return;
-    }
-
-  devname = strdup(buf);
-  if (!devname)
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Out of memory for device name while reading %s\n", path);
-
-      fclose(fp);
-      return;
-    }
-
-  pin = fgets(buf, sizeof(buf), fp);
-  fclose(fp);
-  if (!pin)
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Invalid Remote pairing file %s: no pin\n", path);
-
-      free(devname);
-      return;
-    }
-
-  len = strlen(pin);
-
-  while (len)
-    {
-      if ((buf[len - 1] == '\r') || (buf[len - 1] == '\n'))
-	{
-	  buf[len - 1] = '\0';
-	  len--;
-	}
-      else
-	break;
-    }
-
-  if (len != 4)
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Invalid pin in Remote pairing file %s: pin length should be 4, got %d\n", path, len);
-
-      free(devname);
-      return;
-    }
-
-  pin = strdup(buf);
-  if (!pin)
-    {
-      DPRINTF(E_LOG, L_REMOTE, "Out of memory for device pin while reading %s\n", path);
-
-      free(devname);
-      return;
-    }
-
-  DPRINTF(E_LOG, L_REMOTE, "Read Remote pairing data (name '%s', pin '%s') from %s\n", devname, pin, path);
-
-  pthread_mutex_lock(&remote_lck);
-
-  ret = add_remote_pin_data(devname, pin);
-  free(devname);
-  if (ret < 0)
-    free(pin);
-  else
+  ret = add_remote_pin_data(arglist[0]);
+  if (ret == 0)
     kickoff_pairing();
 
-  pthread_mutex_unlock(&remote_lck);
+  CHECK_ERR(L_REMOTE, pthread_mutex_unlock(&remote_lck));
 }
 
 
@@ -906,9 +743,11 @@ remote_pairing_init(void)
 {
   int ret;
 
-  remote_list = NULL;
+  remote_info = NULL;
 
-#ifdef USE_EVENTFD
+  CHECK_ERR(L_REMOTE, mutex_init(&remote_lck));
+
+#ifdef HAVE_EVENTFD
   pairing_efd = eventfd(0, EFD_CLOEXEC | EFD_NONBLOCK);
   if (pairing_efd < 0)
     {
@@ -933,7 +772,7 @@ remote_pairing_init(void)
 
       return -1;
     }
-#endif /* USE_EVENTFD */
+#endif /* HAVE_EVENTFD */
 
   // No ipv6 for remote at the moment
   ret = mdns_browse("_touch-remote._tcp", AF_INET, touch_remote_cb);
@@ -944,7 +783,7 @@ remote_pairing_init(void)
       goto mdns_browse_fail;
     }
 
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   pairingev = event_new(evbase_main, pairing_efd, EV_READ, pairing_cb, NULL);
 #else
   pairingev = event_new(evbase_main, pairing_pipe[0], EV_READ, pairing_cb, NULL);
@@ -962,7 +801,7 @@ remote_pairing_init(void)
 
  pairingev_fail:
  mdns_browse_fail:
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   close(pairing_efd);
 #else
   close(pairing_pipe[0]);
@@ -976,19 +815,15 @@ remote_pairing_init(void)
 void
 remote_pairing_deinit(void)
 {
-  struct remote_info *ri;
+  if (remote_info)
+    free_remote(remote_info);
 
-  for (ri = remote_list; remote_list; ri = remote_list)
-    {
-      remote_list = ri->next;
-
-      free_remote(ri);
-    }
-
-#ifdef USE_EVENTFD
+#ifdef HAVE_EVENTFD
   close(pairing_efd);
 #else
   close(pairing_pipe[0]);
   close(pairing_pipe[1]);
 #endif
+
+  CHECK_ERR(L_REMOTE, pthread_mutex_destroy(&remote_lck));
 }
