@@ -2,12 +2,12 @@
 #ifndef __SPOTIFY_H__
 #define __SPOTIFY_H__
 
-#include "db.h"
 #include <event2/event.h>
 #include <event2/buffer.h>
+#include <event2/http.h>
 
 int
-spotify_playback_setup(struct media_file_info *mfi);
+spotify_playback_setup(const char *path);
 
 int
 spotify_playback_play();
@@ -27,14 +27,17 @@ spotify_playback_stop_nonblock(void);
 int
 spotify_playback_seek(int ms);
 
-int
-spotify_audio_get(struct evbuffer *evbuf, int wanted);
-
 int
 spotify_artwork_get(struct evbuffer *evbuf, char *path, int max_w, int max_h);
 
 void
-spotify_login(char *path);
+spotify_oauth_interface(struct evbuffer *evbuf, const char *redirect_uri);
+
+void
+spotify_oauth_callback(struct evbuffer *evbuf, struct evkeyvalq *param, const char *redirect_uri);
+
+void
+spotify_login(char **arglist);
 
 int
 spotify_init(void);
