@@ -33,11 +33,7 @@
 #include <libavutil/time.h>
 #include <libavutil/pixdesc.h>
 
-#ifdef HAVE_LIBAVFILTER
-# include <libavfilter/avcodec.h>
-#else
-# include "ffmpeg-compat.h"
-#endif
+#include "ffmpeg-compat.h"
 
 #include "logger.h"
 #include "conffile.h"
@@ -79,6 +75,9 @@ struct decode_ctx {
   // Duration (used to make wav header)
   uint32_t duration;
 
+  // Data kind (used to determine if ICY metadata is relevant to look for)
+  enum data_kind data_kind;
+
   // Contains the most recent packet from av_read_frame
   // Used for resuming after seek and for freeing correctly
   // in transcode_decode()
@@ -402,7 +401,7 @@ encode_write_frame(struct encode_ctx *ctx, AVFrame *filt_frame, unsigned int str
   return ret;
 }
 
-#if defined(HAVE_LIBAV_BUFFERSRC_ADD_FRAME_FLAGS) && defined(HAVE_LIBAV_BUFFERSINK_GET_FRAME)
+#if HAVE_DECL_AV_BUFFERSRC_ADD_FRAME_FLAGS && HAVE_DECL_AV_BUFFERSINK_GET_FRAME
 static int
 filter_encode_write_frame(struct encode_ctx *ctx, AVFrame *frame, unsigned int stream_index)
 {
@@ -580,7 +579,7 @@ flush_encoder(struct encode_ctx *ctx, unsigned int stream_index)
 /* --------------------------- INPUT/OUTPUT INIT --------------------------- */
 
 static int
-open_input(struct decode_ctx *ctx, struct media_file_info *mfi, int decode_video)
+open_input(struct decode_ctx *ctx, const char *path, int decode_video)
 {
   AVDictionary *options;
   AVCodec *decoder;
@@ -597,10 +596,10 @@ open_input(struct decode_ctx *ctx, struct media_file_info *mfi, int decode_video
 
 # ifndef HAVE_FFMPEG
   // Without this, libav is slow to probe some internet streams, which leads to RAOP timeouts
-  if (mfi->data_kind == DATA_KIND_HTTP)
+  if (ctx->data_kind == DATA_KIND_HTTP)
     ctx->ifmt_ctx->probesize = 64000;
 # endif
-  if (mfi->data_kind == DATA_KIND_HTTP)
+  if (ctx->data_kind == DATA_KIND_HTTP)
     av_dict_set(&options, "icy", "1", 0);
 
   // TODO Newest versions of ffmpeg have timeout and reconnect options we should use
@@ -608,14 +607,14 @@ open_input(struct decode_ctx *ctx, struct media_file_info *mfi, int decode_video
   ctx->ifmt_ctx->interrupt_callback.opaque = ctx;
   ctx->timestamp = av_gettime();
 
-  ret = avformat_open_input(&ctx->ifmt_ctx, mfi->path, NULL, &options);
+  ret = avformat_open_input(&ctx->ifmt_ctx, path, NULL, &options);
 
   if (options)
     av_dict_free(&options);
 
   if (ret < 0)
     {
-      DPRINTF(E_LOG, L_XCODE, "Cannot open '%s': %s\n", mfi->path, err2str(ret));
+      DPRINTF(E_LOG, L_XCODE, "Cannot open '%s': %s\n", path, err2str(ret));
       return -1;
     }
 
@@ -628,7 +627,7 @@ open_input(struct decode_ctx *ctx, struct media_file_info *mfi, int decode_video
 
   if (ctx->ifmt_ctx->nb_streams > MAX_STREAMS)
     {
-      DPRINTF(E_LOG, L_XCODE, "File '%s' has too many streams (%u)\n", mfi->path, ctx->ifmt_ctx->nb_streams);
+      DPRINTF(E_LOG, L_XCODE, "File '%s' has too many streams (%u)\n", path, ctx->ifmt_ctx->nb_streams);
       goto out_fail;
     }
 
@@ -636,7 +635,7 @@ open_input(struct decode_ctx *ctx, struct media_file_info *mfi, int decode_video
   stream_index = av_find_best_stream(ctx->ifmt_ctx, AVMEDIA_TYPE_AUDIO, -1, -1, &decoder, 0);
   if ((stream_index < 0) || (!decoder))
     {
-      DPRINTF(E_LOG, L_XCODE, "Did not find audio stream or suitable decoder for %s\n", mfi->path);
+      DPRINTF(E_LOG, L_XCODE, "Did not find audio stream or suitable decoder for %s\n", path);
       goto out_fail;
     }
 
@@ -664,7 +663,7 @@ open_input(struct decode_ctx *ctx, struct media_file_info *mfi, int decode_video
   stream_index = av_find_best_stream(ctx->ifmt_ctx, AVMEDIA_TYPE_VIDEO, -1, -1, &decoder, 0);
   if ((stream_index < 0) || (!decoder))
     {
-      DPRINTF(E_LOG, L_XCODE, "Did not find video stream or suitable decoder for '%s': %s\n", mfi->path, err2str(ret));
+      DPRINTF(E_LOG, L_XCODE, "Did not find video stream or suitable decoder for '%s': %s\n", path, err2str(ret));
       return 0;
     }
 
@@ -857,7 +856,7 @@ close_output(struct encode_ctx *ctx)
   avformat_free_context(ctx->ofmt_ctx);
 }
 
-#ifdef HAVE_LIBAV_GRAPH_PARSE_PTR
+#if HAVE_DECL_AVFILTER_GRAPH_PARSE_PTR
 static int
 open_filter(struct filter_ctx *filter_ctx, AVCodecContext *dec_ctx, AVCodecContext *enc_ctx, const char *filter_spec)
 {
@@ -922,7 +921,6 @@ open_filter(struct filter_ctx *filter_ctx, AVCodecContext *dec_ctx, AVCodecConte
       if (!buffersrc || !buffersink)
 	{
 	  DPRINTF(E_LOG, L_XCODE, "Filtering source or sink element not found\n");
-	  ret = AVERROR_UNKNOWN;
 	  goto out_fail;
 	}
 
@@ -1091,7 +1089,6 @@ open_filter(struct filter_ctx *filter_ctx, AVCodecContext *dec_ctx, AVCodecConte
       if (!buffersrc || !format || !buffersink)
 	{
 	  DPRINTF(E_LOG, L_XCODE, "Filtering source, format or sink element not found\n");
-	  ret = AVERROR_UNKNOWN;
 	  goto out_fail;
 	}
 
@@ -1232,7 +1229,7 @@ close_filters(struct encode_ctx *ctx)
 /*                                  Setup                                    */
 
 struct decode_ctx *
-transcode_decode_setup(struct media_file_info *mfi, int decode_video)
+transcode_decode_setup(enum data_kind data_kind, const char *path, uint32_t song_length, int decode_video)
 {
   struct decode_ctx *ctx;
 
@@ -1243,14 +1240,15 @@ transcode_decode_setup(struct media_file_info *mfi, int decode_video)
       return NULL;
     }
 
-  if (open_input(ctx, mfi, decode_video) < 0)
+  ctx->duration = song_length;
+  ctx->data_kind = data_kind;
+
+  if (open_input(ctx, path, decode_video) < 0)
     {
       free(ctx);
       return NULL;
     }
 
-  ctx->duration = mfi->song_length;
-
   av_init_packet(&ctx->packet);
 
   return ctx;
@@ -1281,7 +1279,8 @@ transcode_encode_setup(struct decode_ctx *src_ctx, enum transcode_profile profil
       return NULL;
     }
 
-  ctx->icy_interval = METADATA_ICY_INTERVAL * ctx->channels * ctx->byte_depth * ctx->sample_rate;
+  if (src_ctx->data_kind == DATA_KIND_HTTP)
+    ctx->icy_interval = METADATA_ICY_INTERVAL * ctx->channels * ctx->byte_depth * ctx->sample_rate;
 
   if (profile == XCODE_PCM16_HEADER)
     {
@@ -1293,7 +1292,7 @@ transcode_encode_setup(struct decode_ctx *src_ctx, enum transcode_profile profil
 }
 
 struct transcode_ctx *
-transcode_setup(struct media_file_info *mfi, enum transcode_profile profile, off_t *est_size)
+transcode_setup(enum data_kind data_kind, const char *path, uint32_t song_length, enum transcode_profile profile, off_t *est_size)
 {
   struct transcode_ctx *ctx;
 
@@ -1304,7 +1303,7 @@ transcode_setup(struct media_file_info *mfi, enum transcode_profile profile, off
       return NULL;
     }
 
-  ctx->decode_ctx = transcode_decode_setup(mfi, profile & XCODE_HAS_VIDEO);
+  ctx->decode_ctx = transcode_decode_setup(data_kind, path, song_length, profile & XCODE_HAS_VIDEO);
   if (!ctx->decode_ctx)
     {
       free(ctx);
@@ -1339,6 +1338,7 @@ transcode_decode_setup_raw(void)
   if (!ctx->ifmt_ctx)
     {
       DPRINTF(E_LOG, L_XCODE, "Out of memory for decode format ctx\n");
+      free(ctx);
       return NULL;
     }
 
@@ -1348,6 +1348,8 @@ transcode_decode_setup_raw(void)
   if (!ctx->audio_stream)
     {
       DPRINTF(E_LOG, L_XCODE, "Could not create stream with PCM16 decoder\n");
+      avformat_free_context(ctx->ifmt_ctx);
+      free(ctx);
       return NULL;
     }
 
@@ -1524,7 +1526,6 @@ transcode_decode(struct decoded_frame **decoded, struct decode_ctx *ctx)
 	  // with empty input until no more frames are returned
 	  DPRINTF(E_DBG, L_XCODE, "Could not read packet, will flush decoders\n");
 
-	  used = 1;
 	  got_frame = flush_decoder(frame, &in_stream, &stream_index, ctx);
 	  if (got_frame)
 	    break;
@@ -1573,19 +1574,22 @@ transcode_decode(struct decoded_frame **decoded, struct decode_ctx *ctx)
     }
   while (!got_frame);
 
-  // Return the decoded frame and stream index
-  *decoded = malloc(sizeof(struct decoded_frame));
-  if (!*decoded)
+  if (got_frame > 0)
     {
-      DPRINTF(E_LOG, L_XCODE, "Out of memory for decoded result\n");
+      // Return the decoded frame and stream index
+      *decoded = malloc(sizeof(struct decoded_frame));
+      if (!(*decoded))
+	{
+	  DPRINTF(E_LOG, L_XCODE, "Out of memory for decoded result\n");
 
-      av_frame_free(&frame);
-      return -1;
+	  av_frame_free(&frame);
+	  return -1;
+	}
+
+      (*decoded)->frame = frame;
+      (*decoded)->stream_index = stream_index;
     }
 
-  (*decoded)->frame = frame;
-  (*decoded)->stream_index = stream_index;
-
   return got_frame;
 }
 
@@ -1630,6 +1634,8 @@ transcode(struct evbuffer *evbuf, int wanted, struct transcode_ctx *ctx, int *ic
   int processed;
   int ret;
 
+  *icy_timer = 0;
+
   processed = 0;
   while (processed < wanted)
     {
@@ -1646,7 +1652,8 @@ transcode(struct evbuffer *evbuf, int wanted, struct transcode_ctx *ctx, int *ic
     }
 
   ctx->encode_ctx->total_bytes += processed;
-  *icy_timer = (ctx->encode_ctx->total_bytes % ctx->encode_ctx->icy_interval < processed);
+  if (ctx->encode_ctx->icy_interval)
+    *icy_timer = (ctx->encode_ctx->total_bytes % ctx->encode_ctx->icy_interval < processed);
 
   return processed;
 }
@@ -1659,10 +1666,17 @@ transcode_raw2frame(uint8_t *data, size_t size)
   int ret;
 
   decoded = malloc(sizeof(struct decoded_frame));
-  frame = av_frame_alloc();
-  if (!decoded || !frame)
+  if (!decoded)
     {
-      DPRINTF(E_LOG, L_XCODE, "Out of memory for decoded struct or frame\n");
+      DPRINTF(E_LOG, L_XCODE, "Out of memory for decoded struct\n");
+      return NULL;
+    }
+
+  frame = av_frame_alloc();
+  if (!frame)
+    {
+      DPRINTF(E_LOG, L_XCODE, "Out of memory for frame\n");
+      free(decoded);
       return NULL;
     }
 
@@ -1682,6 +1696,7 @@ transcode_raw2frame(uint8_t *data, size_t size)
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_XCODE, "Error filling frame with rawbuf: %s\n", err2str(ret));
+      transcode_decoded_free(decoded);
       return NULL;
     }
 
@@ -1776,6 +1791,10 @@ transcode_seek(struct transcode_ctx *ctx, int ms)
   got_pts = av_rescale_q(got_pts, in_stream->time_base, AV_TIME_BASE_Q);
   got_ms = got_pts / (AV_TIME_BASE / 1000);
 
+  // Since negative return would mean error, we disallow it here
+  if (got_ms < 0)
+    got_ms = 0;
+
   DPRINTF(E_DBG, L_XCODE, "Seek wanted %d ms, got %d ms\n", ms, got_ms);
 
   return got_ms;
@@ -1803,24 +1822,3 @@ transcode_metadata(struct transcode_ctx *ctx, int *changed)
   return m;
 }
 
-char *
-transcode_metadata_artwork_url(struct transcode_ctx *ctx)
-{
-  struct http_icy_metadata *m;
-  char *artwork_url;
-
-  if (!ctx->decode_ctx->ifmt_ctx || !ctx->decode_ctx->ifmt_ctx->filename)
-    return NULL;
-
-  artwork_url = NULL;
-
-  m = http_icy_metadata_get(ctx->decode_ctx->ifmt_ctx, 1);
-  if (m && m->artwork_url)
-    artwork_url = strdup(m->artwork_url);
-
-  if (m)
-    http_icy_metadata_free(m, 0);
-
-  return artwork_url;
-}
-
